{
  filename - CTDedicatedDriver.pas

  Purpose:
       This file contains dedicated driver functions for real handler.
}

unit CTDedicatedDriver;
{
History (dd.mm.yy):
1.10.2010 start of development
}

interface

type
  TLogProcedure = procedure(const Text: string) of object; // callback function for logging in higher-level layer

type
  TDedicatedDriver = class
  private
    pLogProcedure: TLogProcedure;

  public
    constructor Create(const pLogProcedureArg: TLogProcedure);
    destructor Destroy; override;
    function HandlerActive: Boolean;
    function MovePusher(const NewPositionIndex, ANumberSites: Integer): Boolean;
    function PullDeviceByPusher: Boolean;
    function DropDeviceByPusher: Boolean;
    function InsertDeviceToSocket(const Index: Integer): Boolean;
    function RemoveDeviceFromSocket(const Index: Integer): Boolean;
  end;

implementation

uses SysUtils;

constructor TDedicatedDriver.Create(const pLogProcedureArg: TLogProcedure);
begin
  inherited Create;

  pLogProcedure := pLogProcedureArg;
end;

destructor TDedicatedDriver.Destroy;
begin

  inherited Destroy;
end;

function TDedicatedDriver.HandlerActive: Boolean;
begin
  pLogProcedure('DedicatedDriver.IsHandlerActive');
  // type source code for implementation of action ReconfigureHandler of dedicated driver
  // ...

  Result := True;
end;

function TDedicatedDriver.MovePusher(const NewPositionIndex, ANumberSites: Integer): Boolean;
begin
  // handler NewPositionIndex is in following order
  //                 |Site #1|Site#2| ... |Site#ANumberSites|   ReelIn   |  ReelOutOK   |  ReelOutBad  |
  //NewPositionIndex |   0   |   1  |     | ANumberSites-1  |ANumberSites|ANumberSites+1|ANumberSites+2|
  pLogProcedure('DedicatedDriver.MovePusher(' + IntToStr(NewPositionIndex) + ', ' + IntToStr(ANumberSites) + ')');

  // type here source code for implementation of action MovePusher of dedicated driver
  // ...

  Result := True;
end;

function TDedicatedDriver.PullDeviceByPusher: Boolean;
begin
  pLogProcedure('DedicatedDriver.PullDeviceByPusher');
  // type here source code for implementation of action PullDeviceByPusher of dedicated driver
  // ...

  Result := True;
end;

function TDedicatedDriver.DropDeviceByPusher: Boolean;
begin
  pLogProcedure('DedicatedDriver.DropDeviceByPusher');
  // type here source code for implementation of action DropDeviceByPusher of dedicated driver
  // ...

  Result := True;
end;

function TDedicatedDriver.InsertDeviceToSocket(const Index: Integer): Boolean;
begin
  pLogProcedure('DedicatedDriver.InsertDeviceToSocket(' + IntToStr(Index + 1) + ')');
  // type here source code for implementation of action InsertDeviceToSocket of dedicated driver
  // ...

  Result := True;
end;

function TDedicatedDriver.RemoveDeviceFromSocket(const Index: Integer): Boolean;
begin
  pLogProcedure('DedicatedDriver.RemoveDeviceFromSocket(' + IntToStr(Index + 1) + ')');
  // type here source code for implementation of action RemoveDeviceFromSocket of dedicated driver
  // ...

  Result := True;
end;

end.
