{
  filename - CTFormVUH.pas

  Purpose:
       This file contains standard application window (form) and main control logic for Virtual Universal Handler (VUH).
}

{$J+,H+}
unit CTFormVUH;
{
History (dd.mm.yy):
21.01.2019 added new command "PROGRAMMING ENVIRONMENT"
10.12.2018 added new commands "SITES STATISTICS", "SITE STATISTICS"
18. 6.2012 added new command "SITE ZIF SOCKET EMPTY"
           checking of TXT_CMD_PROTOCOL_VERSION for required minimum value
21. 3.2012 added new parameters /JobOperatorID:, /JobID:, /ProjectPassword:, /UniqueID: for command "SITES LOAD PROJECT"
           added new results 7, 8, 9 for command "SITES LOAD PROJECT"
           change dialog of project file loading
 6. 2.2012 added new commands "SITES LOAD PROJECT", "APPLICATION CLOSE" and "SITE ACTUATOR STATUS"
           added option to start PG4UWMC remote control by command line parameters /autoconnectremotecontrol, /donotloadproject
17.10.2011 added new commands "SITES STOP" and "SITE MODULE CHECK N"
25. 7.2011 adding support for more as 8 site-s,
           adding new handler speed postition without pusher animation
 1.10.2010 start of development
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Forms, StdCtrls, Contnrs, ComCtrls, ExtCtrls, Controls,
  CTUniHandlerClient, CTDedicatedDriver, CTSprite, CTReelSprite, CTPusherSprite, CTSocketSprite, CTStatistics, CTFormLoadProject,
  Dialogs;

const WM_USER_HANDLER_LOGIC_REALIZATION = WM_USER + 1;
      WM_USER_HANDLER_LOGIC_REALIZATION_FINISHED = WM_USER + 2;

type TSpriteElements = (tSEReelIn = 0, tSEReelOutOK, tSEReelOutBad, tSEPusher, tSESocket);
     TGUIControlState = (tGCSBeforeInitialization = 0, tGCSAfterInitialization, tGCSBeforeStart, tGCSAfterStart);

type
  TFormVUH = class(TForm)
    PanelStartUpServer: TPanel;
    LabelServerPath: TLabel;
    EditServerPath: TEdit;	
    OpenDialogServerPath: TOpenDialog;
    ButtonlServerPath: TButton;    
    CheckBoxAutoConnectRemoteControl: TCheckBox;
    CheckBoxDonotLoadProject: TCheckBox;
    ButtonStartUp: TButton;
    ScrollBoxPaintBox: TScrollBox;
    PaintBox: TPaintBox;
    ImageReelPanelIn: TImage;
    ImageReelPanelOutOK: TImage;
    ImageReelPanelOutBad: TImage;
    ImagePusherEmpty: TImage;	
    ImagePusherFull: TImage;	
    ImageSocketDisabledEmpty: TImage;	
    ImageSocketEnabledEmpty: TImage;	
    ImageSocketEnabledFull: TImage;	
    ImageSocketSideDisabledUp: TImage;
    ImageSocketSideEnabledUp: TImage;	
    ImageSocketSideEnabledDown: TImage;
    PanelControl: TPanel;
    ComboBoxMode: TComboBox;
    ButtonInitialize: TButton;
    ButtonLoadProjectCheckModules: TButton;
    PanelStart: TPanel;
    ButtonStartStop: TButton;
    PanelTotalStop: TPanel;
    ButtonTotalStop: TButton;
    ButtonCloseServerApplication: TButton;
    Label1: TLabel;
    TrackBarSpeed: TTrackBar;		
    RichEditLogWindow: TRichEdit;
    LabelCmdLine: TLabel;
    EditCmdLine: TEdit;    
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormPaint(Sender: TObject);
    procedure PaintBoxPaint(Sender: TObject);
    procedure ButtonlServerPathClick(Sender: TObject);
    procedure ButtonStartUpClick(Sender: TObject);
    procedure ButtonInitializeClick(Sender: TObject);
    procedure ButtonLoadProjectCheckModulesClick(Sender: TObject);
    procedure ButtonStartStopClick(Sender: TObject);
    procedure ButtonTotalStopClick(Sender: TObject);
    procedure ButtonCloseServerApplicationClick(Sender: TObject);
    procedure TrackBarSpeedChange(Sender: TObject);
    procedure EditCmdLineKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);

  private
    { Private declarations }
    UniHandlerClient: TUniHandlerClient; // universal handler
    DedicatedDriver: TDedicatedDriver;
    SpriteList: TObjectList;      // sprite list of zif, input and output tube
    iNumberSites: Integer;        // number of sites of programmer
    iCurrentPosition: Integer;    // position of handler
    BitmapMerged: TBitmap;        // merged bitmap background + pusher, reason: reduce blinking
    iStepSize: Integer;           // define size of step for moving
    iSleepSize: Integer;          // define size of sleep for moving
    bFullManualMode: Boolean;     // for future purpose
    KeyboardCommandsHistory: TStrings;
    FGUIControlState: TGUIControlState;

    FormLoadProject: TFormLoadProject;

    procedure SetHandlerSpeed;
    procedure PaintSocket(const Index: Integer);
    procedure EnableSocket(const Index: Integer);
    procedure DisableSocket(const Index: Integer);
    function GetSocketState(const Index: Integer): TSocketState;
    function MoveReel(const SpriteElement: TSpriteElements): Boolean;
    procedure LedBusy(const Index: Integer; const EmitLight: Boolean; const Redraw: Boolean = True);
    procedure LedGood(const Index: Integer; const EmitLight: Boolean; const Redraw: Boolean = True);
    procedure LedError(const Index: Integer; const EmitLight: Boolean; const Redraw: Boolean = True);
    procedure WriteToLog(const Text: string);
    procedure ResetStatistics;
    procedure IncrementReelStatistic(const SpriteElement: TSpriteElements);
    procedure IncrementSocketStatistic(const Index, ResultCode: Integer);
    procedure Finished(var Message : TMessage); message WM_USER_HANDLER_LOGIC_REALIZATION_FINISHED;
    procedure ReconfigureHandler;
    procedure SetGUIControlState(const AGUIControlState: TGUIControlState);

    // handler logic
    procedure HandlerLogicRealization(var Message : TMessage); message WM_USER_HANDLER_LOGIC_REALIZATION;

    // communication issues
    function SendCommand(const Command: string; var Answer: string; const FromCmdLine: Boolean = False): Boolean; overload;
    function SendCommand(const Command: string; const Site: Integer; var Answer: string): Boolean; overload;

    //// high level operation (Sequences)
    // insert (remove) device to (from) input (output) reel by pusher
    function ReelOperationSequence(const SpriteElement: TSpriteElements): Boolean;
    // insert device to socket by pusher
    function InsertSocketSequence(const Index: Integer): Boolean;
    // remove device from socket by pusher
    function RemoveSocketSequence(const Index: Integer): Boolean;

    //// low level operation
    // move pusher from actual position to required NewPosition
    function MovePusher(const NewPosition, ANumberSites: Integer): Boolean;
    // pull device by pusher
    function PullDeviceByPusher: Boolean;
    // drop device by pusher
    function DropDeviceByPusher: Boolean;
    // insert device to socket
    function InsertDeviceToSocket(const Index: Integer): Boolean;
    // remove device from socket
    function RemoveDeviceFromSocket(const Index: Integer): Boolean;
    // get actuator status
    function GetActuatorStatus(const Index: Integer; const bPaint: Boolean = True): Integer;

  published
    property GUIControlState: TGUIControlState read FGUIControlState write FGUIControlState;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  FormVUH: TFormVUH;

implementation

uses CommCtrl, Types, Math, ShellAPI,
     CTFormOperationalResult;

{$R *.dfm}

const TXT_TRUE = '1';
      TXT_FALSE = '0';

      TXT_CMD_PROTOCOL_VERSION          = 'PROTOCOL VERSION';
      TXT_CMD_PG4UWMC_VERSION           = 'PG4UWMC VERSION';
      TXT_CMD_MAX_INDEX_RANGE           = 'SITES COUNT';
      TXT_CMD_APPLICATION_CLOSE         = 'APPLICATION CLOSE';
      TXT_CMD_PROGRAMMING_ENVIRONMENT   = 'PROGRAMMING ENVIRONMENT';

      TXT_CMD_SITES_STOP                = 'SITES STOP';
      TXT_CMD_SITES_LOAD_PROJECT        = 'SITES LOAD PROJECT';

      TXT_CMD_IDX_SITE_PRESENT          = 'SITE PRESENT';  // state of site = APP_PRESENT and PGM_STATUS
      TXT_CMD_IDX_SITE_BUSY             = 'SITE BUSY';
      TXT_CMD_IDX_SITE_RESULT           = 'SITE RESULT';
      TXT_CMD_IDX_SITE_RESULT_EX        = 'SITE RESULT EX';
      TXT_CMD_IDX_SITE_START            = 'SITE START';
      TXT_CMD_IDX_SITE_START_EX         = 'SITE START EX';
      TXT_CMD_IDX_SITE_SERIALNUMBER     = 'SITE SERIALNUMBER';
      TXT_CMD_IDX_SITE_MODULE_CHECK     = 'SITE MODULE CHECK';
      TXT_CMD_IDX_SITE_ACTUATOR_STATUS  = 'SITE ACTUATOR STATUS';
      TXT_CMD_IDX_SITE_ZIF_SOCKET_EMPTY = 'SITE ZIF SOCKET EMPTY';
      TXT_CMD_IDX_SITES_STATISTICS      = 'SITES STATISTICS';
      TXT_CMD_IDX_SITE_STATISTICS       = 'SITE STATISTICS';

      TXT_CMD_UNKNOWN                   = 'UNKNOWN COMMAND';

      TXT_HANDLER_RUNNING = 'Handler is still running!'#13#10'Please wait a moment while handler correctly stop.';

      TXT_START = 'Start';
      TXT_STOP = 'Stop';

      TXT_MESSAGEBOX_TITLE = 'Virtual Universal Handler';

      DEFAULT_NUMBER_SITES = 8;

      TXT_MINIMUM_REQUIRED_PROTOCOL_VERSION = '1.11';

constructor TFormVUH.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  SpriteList := TObjectList.Create;
  DedicatedDriver := TDedicatedDriver.Create(WriteToLog);

  iNumberSites := DEFAULT_NUMBER_SITES;

  iCurrentPosition := 0;
  iStepSize := 1;
  iSleepSize := 0;

  BitmapMerged := TBitmap.Create;

  KeyboardCommandsHistory := TStringList.Create;

  FGUIControlState := tGCSBeforeInitialization;

  bFullManualMode := False;

  FormLoadProject := TFormLoadProject.Create(Self);
end;

destructor TFormVUH.Destroy;
begin
  SpriteList.Clear;
  SpriteList.Free;

  DedicatedDriver.Free;

  BitmapMerged.Free;

  KeyboardCommandsHistory.Free;

  FormLoadProject.Free;

  inherited Destroy;
end;

procedure TFormVUH.FormShow(Sender: TObject);
begin
  UniHandlerClient := nil;

  // create Background Bitmap
  BitmapMerged.Canvas.Brush.Color := clBtnFace;
  BitmapMerged.Canvas.Font.Name := 'Courier';
  //BitmapMerged.Canvas.Handle := GetDC(PanelHandler.Handle);

  ReconfigureHandler;

  SetGUIControlState(tGCSBeforeInitialization);

  SetHandlerSpeed;
end;

procedure TFormVUH.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if bFullManualMode = False then
    if UniHandlerClient <> nil then
    begin
      UniHandlerClient.Free;
      UniHandlerClient := nil;
    end;
end;

procedure TFormVUH.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose := (GUIControlState = tGCSBeforeInitialization) or (GUIControlState = tGCSAfterInitialization);
  if not CanClose then
    Application.MessageBox(TXT_HANDLER_RUNNING, TXT_MESSAGEBOX_TITLE, MB_OK);
end;

procedure TFormVUH.FormPaint(Sender: TObject);
var i: Integer;
begin
  if (SpriteList.Count <= 0) then
    Exit;

  // draw background
  BitmapMerged.Height := PaintBox.Height;
  BitmapMerged.Width := PaintBox.Width;
  BitmapMerged.Canvas.FillRect(Rect(0, 0, PaintBox.Width, PaintBox.Height));
  // draw sprites
  for i := 0 to SpriteList.Count - 1 do
    (SpriteList[i] as TSprite).Paint(BitmapMerged.Canvas);

  PaintBox.Canvas.Draw(0, 0, BitmapMerged);
end;

procedure TFormVUH.PaintBoxPaint(Sender: TObject);
begin
  Repaint;
end;

procedure TFormVUH.ButtonlServerPathClick(Sender: TObject);
const
  TXT_PG4UWMC = 'PG4UWMC.exe';
begin
  OpenDialogServerPath.Filter := 'Program ' + TXT_PG4UWMC + '|' + TXT_PG4UWMC;
  OpenDialogServerPath.Options := [ofPathMustExist, ofFileMustExist, ofHideReadOnly];
  if OpenDialogServerPath.Execute then
    EditServerPath.Text := OpenDialogServerPath.FileName;
end;

procedure TFormVUH.ButtonStartUpClick(Sender: TObject);

  procedure AddParameterToCommandLine(const NewParameter: string; var Parameters: string);
  begin
    if Parameters <> '' then
      Parameters := Parameters + ' ';
    Parameters := Parameters + NewParameter;
  end;

const
  TXT_CMDLINE_AUTOCONNECTREMOTECONTROL = '/autoconnectremotecontrol';
  TXT_CMDLINE_DONOTLOADPROJECT = '/donotloadproject';
var
  FileNameWithPath, Parameters, sTextTemp: string;
begin
  WriteToLog('Starting remote control server');

  FileNameWithPath := EditServerPath.Text;
  if not FileExists(FileNameWithPath) then
  begin
    sTextTemp := 'Server: ''' + FileNameWithPath + ''' does not exist.' + #13#10 +
                 'Please set correct server with path.';
    WriteToLog(sTextTemp);
    Application.MessageBox(PAnsiChar(sTextTemp), TXT_MESSAGEBOX_TITLE, MB_OK or MB_ICONSTOP);
    Exit;
  end;

  Parameters := '';
  if CheckBoxAutoConnectRemoteControl.State = cbChecked then
    AddParameterToCommandLine(TXT_CMDLINE_AUTOCONNECTREMOTECONTROL, Parameters);
  if CheckBoxDonotLoadProject.State = cbChecked then
    AddParameterToCommandLine(TXT_CMDLINE_DONOTLOADPROJECT, Parameters);

  if ShellExecute(Self.Handle, nil, PAnsiChar(ExtractFileName(FileNameWithPath)), PAnsiChar(Parameters), PAnsiChar(ExtractFilePath(FileNameWithPath)), SW_SHOW) <= 32 then
  begin
    sTextTemp := 'Application: ''' + FileNameWithPath + ''' can not be started.';
    WriteToLog(sTextTemp);
    Application.MessageBox(PAnsiChar(sTextTemp), TXT_MESSAGEBOX_TITLE, MB_OK or MB_ICONSTOP);
    Exit;
  end
  else
    WriteToLog('Application: ''' + FileNameWithPath + ' ' + Parameters + ''' run successfully.');
end;

procedure TFormVUH.ButtonInitializeClick(Sender: TObject);
var TextTemp: string;
    i: Integer;
begin
  if bFullManualMode = False then
  begin
    // stop of universal handler client
    if UniHandlerClient <> nil then
    begin
      UniHandlerClient.Free;
      UniHandlerClient := nil;
      Sleep(100);  // wait for server
    end;

    UniHandlerClient := TUniHandlerClient.Create(WriteToLog);
    if UniHandlerClient.Connect = False then
      Exit;
  end;

  // mandatory as first instruction in code
  if not SendCommand(TXT_CMD_PROTOCOL_VERSION, TextTemp) then
    Exit;
  if TXT_MINIMUM_REQUIRED_PROTOCOL_VERSION > TextTemp then
  begin
    WriteToLog('Minimum required protocol version (' + TXT_MINIMUM_REQUIRED_PROTOCOL_VERSION +
               ') is greater than the current running PG4UWMC protocol version (' + TextTemp + ').');
    WriteToLog('Please close and update PG4UWMC.');
    Exit;
  end;

  if not SendCommand(TXT_CMD_PG4UWMC_VERSION, TextTemp) then
    Exit;

  if not SendCommand(TXT_CMD_MAX_INDEX_RANGE, TextTemp) then
    Exit;

  try
    iNumberSites := StrToInt(TextTemp);
  except
    on EConvertError do iNumberSites := DEFAULT_NUMBER_SITES;
  end;

  ReconfigureHandler;
  for i := 0 to iNumberSites - 1 do
  begin
    // get actuator status
    if GetActuatorStatus(i, False) = -1 then
      Exit;

    // get site presence
    if not SendCommand(TXT_CMD_IDX_SITE_PRESENT, i, TextTemp) then
      Exit;
    if TextTemp <> TXT_TRUE then
      DisableSocket(i)
    else
      // site is present
      EnableSocket(i);

    Application.ProcessMessages;
  end;
  Repaint;

  // get sites serial number
  for i := 0 to iNumberSites - 1 do
  begin
    if not SendCommand(TXT_CMD_IDX_SITE_SERIALNUMBER, i, TextTemp) then
      Exit;
    (SpriteList[i] as TSocketSprite).SerialNumber := TextTemp;
  end;

  // check modules in sites
  for i := 0 to iNumberSites - 1 do
  begin
    if not SendCommand(TXT_CMD_IDX_SITE_MODULE_CHECK, i, TextTemp) then
      Exit;
    Application.ProcessMessages;
  end;

  // check if ZIF socket is empty
  for i := 0 to iNumberSites - 1 do
  begin
    if not SendCommand(TXT_CMD_IDX_SITE_ZIF_SOCKET_EMPTY, i, TextTemp) then
      Exit;
    Application.ProcessMessages;
  end;

  Repaint;

  if DedicatedDriver.HandlerActive then
    SetGUIControlState(tGCSAfterInitialization);
end;

procedure TFormVUH.ButtonLoadProjectCheckModulesClick(Sender: TObject);

  procedure AddNoneBlankValueParameterToCommandLine(const NewParameter, ValueOfNewParameter: string; var Parameters: string);
  begin
    if ValueOfNewParameter <> '' then
    begin
      if Parameters <> '' then
        Parameters := Parameters + ' ';
      Parameters := Parameters + NewParameter + ValueOfNewParameter;
    end;
  end;

  function ReportProgrammingEnvironment(Text: string): string;
  var
    iIndexInText: Integer;
    sItem: string;
  begin
    Result := 'Report of programming environment:' + #13#10;

    repeat
      iIndexInText := Pos('|', Text);
      if (iIndexInText > 0) then
      begin
        sItem := Copy(Text, 1, iIndexInText - 1);
        Delete(Text, 1, iIndexInText);
        Result := Result + sItem + #13#10;
      end;
    until Text = '';
  end;

var ProjectFileNameWithPath, sAnswer, sTextTemp: string;
    i, iAnswer: integer;
    CheckModules: Boolean;
    OptionalParamaters, JobOperatorID, JobID, ProjectPassword, UniqueID: string;
begin
  if FormLoadProject.ShowModal = mrOk then
  begin
    FormLoadProject.GetParameter(CheckModules, ProjectFileNameWithPath, JobOperatorID, JobID, ProjectPassword, UniqueID);
    WriteToLog('Load project: ' + ProjectFileNameWithPath);

    OptionalParamaters := '';
    AddNoneBlankValueParameterToCommandLine('/JobOperatorID:', JobOperatorID, OptionalParamaters);
    AddNoneBlankValueParameterToCommandLine('/JobID:', JobID, OptionalParamaters);
    AddNoneBlankValueParameterToCommandLine('/ProjectPassword:', ProjectPassword, OptionalParamaters);
    AddNoneBlankValueParameterToCommandLine('/UniqueID:', UniqueID, OptionalParamaters);
    if OptionalParamaters <> '' then
      OptionalParamaters := ' ' + OptionalParamaters;
    if not SendCommand(TXT_CMD_SITES_LOAD_PROJECT + ' ' + ProjectFileNameWithPath + OptionalParamaters, sAnswer) then
      Exit;

    // convert answer from string to integer
    try
      iAnswer := StrToInt(sAnswer);
    except
      on EConvertError do iAnswer := 0;
    end;

    // process answer of command
    case iAnswer of
      0: sTextTemp := 'Error: Unknown.';
      1: sTextTemp := 'Project file load successful.';
      2: sTextTemp := 'Error: Check ''Use Site #1 project for all Sites'' in ''PG4UWMC Settings'' dialog.';
      3: sTextTemp := 'Error: Some site(s) is busy, stop programming operation before loading.';
      4: sTextTemp := 'Error: Master site was not correctly started.' + #13#10+
                      '       Try to do following steps in PG4UWMC:' + #13#10+
                      '         1. disconnect Sites' + #13#10+
                      '         2. connect Sites' + #13#10+
                      '         3. load project.';
      5: sTextTemp := 'Error: Failure during loading process.';
      6: sTextTemp := 'Error: Bad project file password.';
      7: sTextTemp := 'Error: Project load forbidden.';
      8: sTextTemp := 'Project file load successful, but forced default "Program" operation.';
      9: sTextTemp := 'Error: Bad project file unique ID.';
    else
      sTextTemp := 'Error: Unknown.';
    end;
    WriteToLog(sTextTemp);
    if (iAnswer <> 1) and (iAnswer <> 8) then
    begin
      Application.MessageBox(PAnsiChar(sTextTemp), TXT_MESSAGEBOX_TITLE, MB_OK or MB_ICONWARNING);
      Exit;
    end
    else
    begin
      // read and report programming environment
      if SendCommand(TXT_CMD_PROGRAMMING_ENVIRONMENT, sAnswer) then
        WriteToLog(ReportProgrammingEnvironment(sAnswer));
    end;

    if CheckModules then
    begin
      // check modules in sites
      sTextTemp := '';
      for i := 0 to iNumberSites - 1 do
      begin
        if not SendCommand(TXT_CMD_IDX_SITE_MODULE_CHECK, i, sAnswer) then
          Exit;
        if sAnswer <> TXT_TRUE then
          if sTextTemp <> '' then
            sTextTemp := sTextTemp + ', #' + IntToStr(i + 1)
          else
            sTextTemp := sTextTemp + '#' + IntToStr(i + 1);
        Application.ProcessMessages;
      end;
      if sTextTemp <> '' then
      begin
        sTextTemp := 'AP module(s) errors summary: ' + sTextTemp + '.';
        WriteToLog(sTextTemp);
        Application.MessageBox(PAnsiChar(sTextTemp), TXT_MESSAGEBOX_TITLE, MB_OK or MB_ICONWARNING);
      end
      else
        WriteToLog('AP module(s) check successful.');
    end;
  end;
end;

procedure TFormVUH.ButtonStartStopClick(Sender: TObject);
begin
  if GUIControlState = tGCSAfterInitialization then
  begin
    SetGUIControlState(tGCSBeforeStart);

    // run Handler Logic Realization
    PostMessage(Handle, WM_USER_HANDLER_LOGIC_REALIZATION, 0, 0);
  end
  else
    SetGUIControlState(tGCSAfterStart);
end;

procedure TFormVUH.ButtonTotalStopClick(Sender: TObject);
var sAnswer: string;
begin
  WriteToLog('Total-Stop button pressed.');
  if not SendCommand(TXT_CMD_SITES_STOP, sAnswer) then
    Exit;
end;

procedure TFormVUH.ButtonCloseServerApplicationClick(Sender: TObject);
var sAnswer, sTextTemp: string;
    iAnswer: integer;
begin
  WriteToLog('Close Application button pressed.');
  if not SendCommand(TXT_CMD_APPLICATION_CLOSE, sAnswer) then
    Exit;

  // convert answer from string to integer
  try
    iAnswer := StrToInt(sAnswer);
  except
    on EConvertError do iAnswer := 0;
  end;

  // process answer of command
  case iAnswer of
    0: sTextTemp := 'Error: Unknown.';
    1: sTextTemp := 'Close application successful.';
    2: sTextTemp := 'Error: Some site(s) is busy.';
    3: sTextTemp := 'Error: Impossible to disconnect site(s).';
  else
    sTextTemp := 'Error: Unknown.';
  end;
  WriteToLog(sTextTemp);
  if iAnswer <> 1 then
    Application.MessageBox(PAnsiChar(sTextTemp), TXT_MESSAGEBOX_TITLE, MB_OK or MB_ICONWARNING);
end;

procedure TFormVUH.TrackBarSpeedChange(Sender: TObject);
begin
  SetHandlerSpeed;
end;

procedure TFormVUH.EditCmdLineKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
const constIndexInKeyboardCommandsHistory: Integer = 0;
var sAnswer: string;
begin
  // set last command when UP ARROW key is press
  if Key = VK_UP then
  begin
    // select previous command from history
    if KeyboardCommandsHistory.Count > 0 then
      if constIndexInKeyboardCommandsHistory < KeyboardCommandsHistory.Count then
      begin
        Inc(constIndexInKeyboardCommandsHistory);
        EditCmdLine.Text := KeyboardCommandsHistory[constIndexInKeyboardCommandsHistory - 1];
      end;
    EditCmdLine.SelStart := Length(EditCmdLine.Text);
  end
  // set last command when DOWN ARROW key is press
  else if Key = VK_DOWN then
  begin
    // select following command from history
    if KeyboardCommandsHistory.Count > 0 then
      if constIndexInKeyboardCommandsHistory > 1 then
      begin
        Dec(constIndexInKeyboardCommandsHistory);
        EditCmdLine.Text := KeyboardCommandsHistory[constIndexInKeyboardCommandsHistory - 1];
      end;
    EditCmdLine.SelStart := Length(EditCmdLine.Text);
  end
  else if Key = VK_RETURN then
  begin
    if not SendCommand(EditCmdLine.Text, sAnswer, True) then
      ;

    // insert command to history
    if KeyboardCommandsHistory.Count = 0 then
      KeyboardCommandsHistory.Add(EditCmdLine.Text)
    else if KeyboardCommandsHistory[0] <> EditCmdLine.Text then
      KeyboardCommandsHistory.Insert(0, EditCmdLine.Text);
    constIndexInKeyboardCommandsHistory := 0;

    // clear cmd line
    EditCmdLine.Text := '';
  end;
end;

procedure TFormVUH.SetHandlerSpeed;
begin
  case TrackBarSpeed.Position of
    1: begin iStepSize := 1; iSleepSize := 2; end;
    2: begin iStepSize := 1; iSleepSize := 1; end;
    3: begin iStepSize := 1; iSleepSize := 0; end;
    4: begin iStepSize := 2; iSleepSize := 0; end;
    5: begin iStepSize := 4; iSleepSize := 0; end;
    6: begin iStepSize := 5; iSleepSize := 0; end;    
  else
    iStepSize := 1; iSleepSize := 0;
  end;
  (SpriteList[iNumberSites + Ord(tSEReelIn)] as TReelSprite).SetDeltaMoving(iStepSize);
  (SpriteList[iNumberSites + Ord(tSEReelOutOK)] as TReelSprite).SetDeltaMoving(iStepSize);
  (SpriteList[iNumberSites + Ord(tSEReelOutBad)] as TReelSprite).SetDeltaMoving(iStepSize);
end;

procedure TFormVUH.PaintSocket(const Index: Integer);
begin
  (SpriteList[Index] as TSocketSprite).Paint(BitmapMerged.Canvas);
  (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).Paint(BitmapMerged.Canvas);
  PaintBox.Canvas.Draw(0, 0, BitmapMerged);
end;

procedure TFormVUH.EnableSocket(const Index: Integer);
begin
  if (SpriteList[Index] as TSocketSprite).SocketState = tSDisabledEmpty then
  begin
    (SpriteList[Index] as TSocketSprite).SocketState := tSEnabledEmpty;
    PaintSocket(Index);
  end;
end;

procedure TFormVUH.DisableSocket(const Index: Integer);
begin
  if (SpriteList[Index] as TSocketSprite).SocketState = tSDisabledEmpty then
    Exit;

  (SpriteList[Index] as TSocketSprite).SocketState := tSDisabledEmpty;
  LedBusy(Index, False, False);
  LedGood(Index, False, False);
  LedError(Index, False, False);
  PaintSocket(Index);
end;

function TFormVUH.GetSocketState(const Index: Integer): TSocketState;
begin
  Result := (SpriteList[Index] as TSocketSprite).SocketState;
end;

function TFormVUH.MoveReel(const SpriteElement: TSpriteElements): Boolean;
begin
  Result := False;

  if not (SpriteElement in [tSEReelIn, tSEReelOutOK, tSEReelOutBad]) then
    Exit;

  (SpriteList[iNumberSites + Ord(SpriteElement)] as TReelSprite).MoveFrame;
  Result := True;
end;

procedure TFormVUH.LedBusy(const Index: Integer; const EmitLight: Boolean; const Redraw: Boolean);
begin
  (SpriteList[Index] as TSocketSprite).LedBusy := EmitLight;
  if Redraw then
    PaintSocket(Index);
end;

procedure TFormVUH.LedGood(const Index: Integer; const EmitLight: Boolean; const Redraw: Boolean);
begin
  (SpriteList[Index] as TSocketSprite).LedGood := EmitLight;
  if Redraw then
    PaintSocket(Index);
end;

procedure TFormVUH.LedError(const Index: Integer; const EmitLight: Boolean; const Redraw: Boolean);
begin
  (SpriteList[Index] as TSocketSprite).LedError := EmitLight;
  if Redraw then
    PaintSocket(Index);
end;

procedure TFormVUH.WriteToLog(const Text: string);
begin
  RichEditLogWindow.Lines.Add(Text);
end;

procedure TFormVUH.ResetStatistics;
var i: Integer;
begin
  (SpriteList[iNumberSites + Ord(tSEReelIn)] as TReelSprite).Statistics := 0;
  (SpriteList[iNumberSites + Ord(tSEReelOutOK)] as TReelSprite).Statistics := 0;
  (SpriteList[iNumberSites + Ord(tSEReelOutBad)] as TReelSprite).Statistics := 0;

  for i := 0 to iNumberSites - 1 do
    (SpriteList[i] as TSocketSprite).Statistics.Reset;
end;

procedure TFormVUH.IncrementReelStatistic(const SpriteElement: TSpriteElements);
begin
  if not (SpriteElement in [tSEReelIn, tSEReelOutOK, tSEReelOutBad]) then
    Exit;

  (SpriteList[iNumberSites + Ord(SpriteElement)] as TReelSprite).Statistics :=
    (SpriteList[iNumberSites + Ord(SpriteElement)] as TReelSprite).Statistics + 1;
end;

procedure TFormVUH.IncrementSocketStatistic(const Index, ResultCode: Integer);
begin
  (SpriteList[Index] as TSocketSprite).Statistics.IncrementByResult(ResultCode);
end;

procedure TFormVUH.Finished(var Message : TMessage);
var bCorrectFinish: Boolean;
begin
  bCorrectFinish := Message.WParam <> 0;
  if bCorrectFinish then
    SetGUIControlState(tGCSAfterInitialization)
  else
    SetGUIControlState(tGCSBeforeInitialization);
end;

procedure TFormVUH.ReconfigureHandler;
const X0 = 10;
      Y0 = 10;
var i: Integer;
    SizeHorizontalElement: Single;
begin
  SpriteList.Clear;

  SizeHorizontalElement := ImageSocketEnabledEmpty.Picture.Width * 1.3;

  PaintBox.Width := X0 + Trunc(Max(iNumberSites, 8) * SizeHorizontalElement);
  Self.Constraints.MaxWidth := Self.Width - ScrollBoxPaintBox.Width + PaintBox.Width + (2 * GetSystemMetrics(SM_CXEDGE));
  ScrollBoxPaintBox.Repaint;

  // adding sites to list
  for i := 0 to iNumberSites - 1 do
    SpriteList.Add(TSocketSprite.Create(X0 + Trunc(i * SizeHorizontalElement), 20 * Y0, i + 1,
                   ImageSocketDisabledEmpty.Picture.Bitmap, ImageSocketEnabledEmpty.Picture.Bitmap, ImageSocketEnabledFull.Picture.Bitmap,
                   ImageSocketSideDisabledUp.Picture.Bitmap, ImageSocketSideEnabledUp.Picture.Bitmap, ImageSocketSideEnabledDown.Picture.Bitmap));

  SizeHorizontalElement := ImageReelPanelIn.Picture.Width * 3.0 / 4.0 * 1.2;
  // adding other elements to list
  // ReelIn
  SpriteList.Add(TReelSprite.Create(X0 + Trunc(Ord(tSEReelIn) * SizeHorizontalElement), 1 * Y0, ImageReelPanelIn.Picture.Bitmap));
  // ReelOutOK
  SpriteList.Add(TReelSprite.Create(X0 + Trunc(Ord(tSEReelOutOK) * SizeHorizontalElement), 1 * Y0, ImageReelPanelOutOK.Picture.Bitmap));
  // ReelOutBad
  SpriteList.Add(TReelSprite.Create(X0 + Trunc(Ord(tSEReelOutBad) * SizeHorizontalElement), 1 * Y0, ImageReelPanelOutBad.Picture.Bitmap));
  // Pusher
  SpriteList.Add(TPusherSprite.Create(X0 + Trunc(Ord(tSEReelIn) * SizeHorizontalElement), 1 * Y0, ImagePusherEmpty.Picture.Bitmap, ImagePusherFull.Picture.Bitmap));

  ResetStatistics;

  iCurrentPosition := iNumberSites + Ord(tSEReelIn);
end;

procedure TFormVUH.SetGUIControlState(const AGUIControlState: TGUIControlState);
begin
  case AGUIControlState of
    tGCSBeforeInitialization: begin
                          ComboBoxMode.Enabled := True;

                          ButtonInitialize.Enabled := True;

                          ButtonStartStop.Caption := TXT_START;
                          ButtonStartStop.Enabled := False;
                          PanelStart.Color := clGreen;

                          ButtonLoadProjectCheckModules.Enabled := False;
                          ButtonTotalStop.Enabled := False;
                          ButtonCloseServerApplication.Enabled := False;
                          PanelTotalStop.Color := clMaroon;

                          LabelCmdLine.Enabled := False;
                          EditCmdLine.Enabled := False;
                        end;
    tGCSAfterInitialization: begin
                          ComboBoxMode.Enabled := True;

                          ButtonInitialize.Enabled := True;

                          ButtonStartStop.Caption := TXT_START;
                          ButtonStartStop.Enabled := True;
                          PanelStart.Color := clLime;

                          ButtonLoadProjectCheckModules.Enabled := True;
                          ButtonTotalStop.Enabled := True;
                          ButtonCloseServerApplication.Enabled := True;
                          PanelTotalStop.Color := clRed;

                          LabelCmdLine.Enabled := True;
                          EditCmdLine.Enabled := True;
                        end;
    tGCSBeforeStart: begin
                          ComboBoxMode.Enabled := False;

                          ButtonInitialize.Enabled := False;

                          ButtonStartStop.Caption := TXT_STOP;
                          ButtonStartStop.Enabled := True;
                          PanelStart.Color := clLime;

                          ButtonLoadProjectCheckModules.Enabled := False;
                          ButtonTotalStop.Enabled := True;
                          ButtonCloseServerApplication.Enabled := False;
                          PanelTotalStop.Color := clRed;

                          LabelCmdLine.Enabled := False;
                          EditCmdLine.Enabled := False;
                     end;
    tGCSAfterStart: begin
                          ComboBoxMode.Enabled := False;

                          ButtonInitialize.Enabled := False;

                          ButtonStartStop.Caption := TXT_STOP;
                          ButtonStartStop.Enabled := False;
                          PanelStart.Color := clGreen;

                          ButtonLoadProjectCheckModules.Enabled := False;
                          ButtonTotalStop.Enabled := True;
                          ButtonCloseServerApplication.Enabled := False;
                          PanelTotalStop.Color := clRed;

                          LabelCmdLine.Enabled := False;
                          EditCmdLine.Enabled := False;
                    end;
  else
    Exit;
  end;
  FGUIControlState := AGUIControlState;
end;

procedure TFormVUH.HandlerLogicRealization(var Message : TMessage);

  function SocketsEmpty: Boolean;
  var i: Integer;
  begin
    Result := True;

    for i := 0 to iNumberSites - 1 do
      if GetSocketState(i) = tSEnabledFull then
      begin
        Result := False;

        Break;
      end;
  end;

  function ReportStatistics(Text: string): string;
  var
    iIndexInText, iStatisticValue: Integer;
    sStatisticItem: string;
    iStatisticSuccess, iStatisticAdapterTestFailure, iStatisticInsertionTestFailure, iStatisticIDCheckFailure,
    iStatisticAccessCanceledByUserFailure, iStatisticOtherFailure, iStatisticOperationalFail: Integer;
  begin
    iStatisticSuccess := 0;
    iStatisticAdapterTestFailure := 0;
    iStatisticInsertionTestFailure := 0;
    iStatisticIDCheckFailure := 0;
    iStatisticAccessCanceledByUserFailure := 0;
    iStatisticOtherFailure := 0;
    iStatisticOperationalFail := 0;

    repeat
      iIndexInText := Pos('|', Text);
      if (iIndexInText > 0) then
      begin
        sStatisticItem := Copy(Text, 1, iIndexInText - 1);
        Delete(Text, 1, iIndexInText);

        iIndexInText := Pos(':', sStatisticItem);
        if (iIndexInText > 0) then
        begin
          iStatisticValue := StrToInt(Copy(sStatisticItem, iIndexInText + 1, Length(sStatisticItem)));
          case StrToInt(Copy(sStatisticItem, 1, iIndexInText - 1)) of
            0: begin
                 Result := ' Unavailable statistics.';
                 Exit;
               end;
            1: iStatisticSuccess := iStatisticValue;
            2: iStatisticAdapterTestFailure := iStatisticValue;
            3: iStatisticInsertionTestFailure := iStatisticValue;
            4: iStatisticIDCheckFailure := iStatisticValue;
            5: iStatisticAccessCanceledByUserFailure := iStatisticValue;
            6: iStatisticOtherFailure := iStatisticValue;
          else
            iStatisticOperationalFail := iStatisticOperationalFail + iStatisticValue;
          end;
        end;

      end;
    until Text = '';

    Result := ' Success:' + IntToStr(iStatisticSuccess) +
              '  Oper. failure(s):' + IntToStr(iStatisticOperationalFail) +
              '  Other failure(s):' + IntToStr(iStatisticAdapterTestFailure +
                                               iStatisticInsertionTestFailure +
                                               iStatisticIDCheckFailure +
                                               iStatisticAccessCanceledByUserFailure +
                                               iStatisticOtherFailure) +
              '  Total:' + IntToStr(iStatisticSuccess +
                                    iStatisticAdapterTestFailure +
                                    iStatisticInsertionTestFailure +
                                    iStatisticIDCheckFailure +
                                    iStatisticAccessCanceledByUserFailure +
                                    iStatisticOtherFailure +
                                    iStatisticOperationalFail);
  end;


const bSemaphore: Boolean = True; // indicate running activity of procedure, protect procedure from multiple calling
var i: Integer;
    LoadDevice: Integer;
    iResultCode: Integer;
    sCommandAnswer: string;
    bInitialSocketsLoading: Boolean;
    bCorrectFinish: Boolean;
    FormOperationalResult: TFormOperationalResult;
    PointTemp: TPoint;
begin
  if not bSemaphore then
    Exit;

  bSemaphore := False;
  bCorrectFinish := False;

  // reset result flag for all sites
  // operation over each site of programmer
  for i := 0 to iNumberSites - 1 do
    // CHECK RESULT OF SITE
    if not SendCommand(TXT_CMD_IDX_SITE_RESULT, i, sCommandAnswer) then
      Exit;

  iResultCode := 0; // only for delphi language
  try
    // go to new device position
    if not MovePusher(iNumberSites + Ord(tSEReelIn), iNumberSites) then Exit;

    // signalize request for initial loading of devices to sockets
    bInitialSocketsLoading := True;

    LoadDevice := 0;

    // infinity main loop
    while (GUIControlState = tGCSBeforeStart) or (not SocketsEmpty) do
    begin
      Application.ProcessMessages;

      // operation over each site of programmer
      for i := 0 to iNumberSites - 1 do
      begin
        // CHECK IF SITE IS PRESENT
        if not SendCommand(TXT_CMD_IDX_SITE_PRESENT, i, sCommandAnswer) then
          Exit;
        if sCommandAnswer <> TXT_TRUE then
        begin
          DisableSocket(i);

          Continue;
        end;
        // site is present


        EnableSocket(i);

        // CHECK IF SITE IS BUSY
        if not SendCommand(TXT_CMD_IDX_SITE_BUSY, i, sCommandAnswer) then
          Exit;
        if sCommandAnswer <> TXT_FALSE then
        begin
          LedBusy(i, True); LedGood(i, False); LedError(i, False);

          Continue;
        end;
        // site is not busy
        LedBusy(i, False);

        // CHECK RESULT OF SITE
        if not SendCommand(TXT_CMD_IDX_SITE_RESULT, i, sCommandAnswer) then
          Exit;

        try
          iResultCode := StrToInt(sCommandAnswer);
        except
          on EConvertError do iResultCode := 0;
        end;

        // load device
        if bFullManualMode then
          if (GUIControlState = tGCSBeforeStart) and (LoadDevice + 1 <= iNumberSites) then
          begin
            Inc(LoadDevice);
            iResultCode := 0;
          end;

        if bInitialSocketsLoading then
        begin
          // site result ... Unavailable
          if iResultCode <> 0 then
            Continue;
        end
        else // if bInitialSocketsLoading then
        begin
          // INCREMENT STATISTICS, REMOVE DEVICE FROM SOCKET AND INSERT IT TO OUTPUT REEL
          // site result ... Unavailable
          if iResultCode = 0 then
            Continue;

          case GetActuatorStatus(i) of
            0: ;
            1: ;
          else
            //Continue;
          end;

          if ComboBoxMode.ItemIndex = 1 then
          begin
            FormOperationalResult := TFormOperationalResult.Create(Self);

            PointTemp.X := PanelControl.Left + 1;
            PointTemp.Y := PanelControl.Top + 1;
            PointTemp := Self.ClientToScreen(PointTemp);
            FormOperationalResult.Left := PointTemp.X;
            FormOperationalResult.Top := PointTemp.Y;

            if not MovePusher(i, iNumberSites) then Exit;

            FormOperationalResult.SetButtonFocus(iResultCode = 1);
            if FormOperationalResult.ShowModal = mrYes then
              iResultCode := 1 // Good
            else
              iResultCode := 7; // Operational Fail

            FormOperationalResult.Release;
          end;

          IncrementSocketStatistic(i, iResultCode);
          case iResultCode of
            // site result ... Good
            1:  begin
                  LedGood(i, True); LedError(i, False);
                end
          else
            // site result ... Error
            LedGood(i, False); LedError(i, True);
          end;

          // get statistics for site to log
          if not SendCommand(TXT_CMD_IDX_SITE_STATISTICS, i, sCommandAnswer) then
            Exit
          else
            WriteToLog('Statistics info for Site#' + IntToStr(i + 1) + ReportStatistics(sCommandAnswer));

          if GetSocketState(i) = tSEnabledFull then
          begin
            if not RemoveSocketSequence(i) then Exit;

            case iResultCode of
              // site result ... Good
              1:  if not ReelOperationSequence(tSEReelOutOK) then Exit;
            else
              // site result ... Error
              if not ReelOperationSequence(tSEReelOutBad) then Exit;
            end;
          end
          else
            Exit;
        end; // else // if bInitialSocketsLoading then


        // TAKE NEW DEVICE, INSERT IT TO SOCKET AND START SITE
        if (GUIControlState = tGCSBeforeStart) then
        begin
          if not (GetSocketState(i) = tSEnabledEmpty) then Exit;

          if not ReelOperationSequence(tSEReelIn) then Exit;

          if not InsertSocketSequence(i) then Exit;

          // start site #
          if not SendCommand(TXT_CMD_IDX_SITE_START, i, sCommandAnswer) then
          //if not SendCommand(TXT_CMD_IDX_SITE_START_EX + ' ' + CustomID:CompanyGeneratedID' + '|', i, sCommandAnswer) then
            Exit;
          GetActuatorStatus(i);
          if sCommandAnswer = TXT_FALSE then
            Continue;

          // CHECK IF SITE IS BUSY (Busy Led correction)
          if not SendCommand(TXT_CMD_IDX_SITE_BUSY, i, sCommandAnswer) then
            Exit;
          if sCommandAnswer = TXT_FALSE then
          begin
            LedBusy(i, True); LedGood(i, False); LedError(i, False);
          end;
        end;
      end;  // for i := 0 to iNumberSites - 1 do
      bInitialSocketsLoading := False;
    end; // while (GUIControlState = tGCSBeforeStart) or (not SocketsEmpty) do

    // go to new device position
    if not MovePusher(iNumberSites + Ord(tSEReelIn), iNumberSites) then Exit;

    // get total statistics to log
    if not SendCommand(TXT_CMD_IDX_SITES_STATISTICS, sCommandAnswer) then
      Exit
    else
      WriteToLog('Summary:' + ReportStatistics(sCommandAnswer));

    bCorrectFinish := True;

  finally
    if not bCorrectFinish then
      WriteToLog('Error: Incorrect end of operation, analyse situation and continue by (Re)Initialization (recommended).');

    PostMessage(Handle, WM_USER_HANDLER_LOGIC_REALIZATION_FINISHED, Ord(bCorrectFinish), 0);
    bSemaphore := True;
  end;
end;

function TFormVUH.SendCommand(const Command: string; var Answer: string; const FromCmdLine: Boolean = False): Boolean;
begin
  Result := False;

  if UniHandlerClient = nil then
  begin
    WriteToLog('Command: ' + Command + ' not accepted, (Re)Initialization is required.');
    Exit;
  end;

  if UniHandlerClient.Write(Command) then
    if UniHandlerClient.Read(Answer) then
    begin
      WriteToLog(Command + ' ... ' + Answer);

      // successful finish
      Result := True;
      Exit;
    end
    else
    begin
      if Answer <> '' then
        WriteToLog(Command + ' ... ' + Answer);
    end;

  if not FromCmdLine then
    // incorrect finish
    SetGUIControlState(tGCSBeforeInitialization);
end;

function TFormVUH.SendCommand(const Command: string; const Site: Integer; var Answer: string): Boolean;
const TXT_SPACE = ' ';
begin
  Result := SendCommand(Command + TXT_SPACE + IntToStr(Site + 1), Answer);
end;

// remove (insert) device from (to) input (output) reel by pusher
function TFormVUH.ReelOperationSequence(const SpriteElement: TSpriteElements): Boolean;
begin
  Result := False;

  if not (SpriteElement in [tSEReelIn, tSEReelOutOK, tSEReelOutBad]) then Exit;

  if not MovePusher(iNumberSites + Ord(SpriteElement), iNumberSites) then Exit;

  case SpriteElement of
    tSEReelIn: if not PullDeviceByPusher then Exit;
    tSEReelOutOK, tSEReelOutBad: if not DropDeviceByPusher then Exit;
  end;

  if not MoveReel(SpriteElement) then Exit;
  IncrementReelStatistic(SpriteElement);

  Result := True;
end;

// insert device to socket by pusher
function TFormVUH.InsertSocketSequence(const Index: Integer): Boolean;
begin
  Result := False;

  // go to Index site
  if not MovePusher(Index, iNumberSites) then Exit;
  if not InsertDeviceToSocket(Index) then Exit;
  if not DropDeviceByPusher then Exit;

  Result := True;
end;

// remove device from socket by pusher
function TFormVUH.RemoveSocketSequence(const Index: Integer): Boolean;
begin
  Result := False;

  // go to Index site
  if not MovePusher(Index, iNumberSites) then Exit;
  if not RemoveDeviceFromSocket(Index) then Exit;
  if not PullDeviceByPusher then Exit;

  Result := True
end;

// move pusher from actual position to required NewPosition
function TFormVUH.MovePusher(const NewPosition, ANumberSites: Integer): Boolean;
var iCounter, PositionX, PositionY, iDelta, iDeltaAxisX, iDeltaAxisY, iDirectionAxisX, iDirectionAxisY: Longint;
    PointFrom, PointTo: TPoint;

  function GetPosition(const Index: Integer): TPoint;
  begin
    if (Index - ANumberSites) in [Ord(tSEReelIn), Ord(tSEReelOutOK), Ord(tSEReelOutBad)] then
      Result := (SpriteList[Index] as TReelSprite).GetCentrePosition((SpriteList[ANumberSites + Ord(tSEPusher)] as TPusherSprite).GetWidth)
    else
      Result := (SpriteList[Index] as TSprite).GetPosition;
  end;

  procedure PaintSprites;
  var i: Integer;
  begin
    // draw background
    BitmapMerged.Canvas.FillRect(Rect(0, 0, PaintBox.Width, PaintBox.Height));
    // draw sprites
    for i := 0 to SpriteList.Count - 1 do
      (SpriteList[i] as TSprite).Paint(BitmapMerged.Canvas);
    PaintBox.Canvas.Draw(0, 0, BitmapMerged);
  end;

begin
  Result := False;

  if (SpriteList.Count <= 0) then
    Exit;

  if not DedicatedDriver.MovePusher(NewPosition, iNumberSites) then Exit;

  if NewPosition <> iCurrentPosition then
  begin
    PointFrom := GetPosition(iCurrentPosition);
    PointTo   := GetPosition(NewPosition);

    if iStepSize <> 5 then
    begin
      // x,y axis movement
      iDeltaAxisX := Abs(PointFrom.X - PointTo.X);
      iDeltaAxisY := Abs(PointFrom.Y - PointTo.Y);
      iDelta := Max(iDeltaAxisX, iDeltaAxisY);
      if (PointTo.X - PointFrom.X) > 0 then
        iDirectionAxisX := 1
      else
        iDirectionAxisX := -1;
      if (PointTo.Y - PointFrom.Y) > 0 then
        iDirectionAxisY := 1
      else
        iDirectionAxisY := -1;

      iCounter := 0;
      while (iCounter < iDelta) do
      begin
        Application.ProcessMessages;
        Inc(iCounter, iStepSize);
        if iDelta = iDeltaAxisX then
          PositionX := PointFrom.X + (iCounter * iDirectionAxisX)
        else
          PositionX := PointFrom.X + ((iCounter * iDirectionAxisX * iDeltaAxisX) div iDelta);
        if iDelta = iDeltaAxisY then
          PositionY := PointFrom.Y + (iCounter * iDirectionAxisY)
        else
          PositionY := PointFrom.Y + ((iCounter * iDirectionAxisY * iDeltaAxisY) div iDelta);

        // set pusher position
        (SpriteList[ANumberSites + Ord(tSEPusher)] as TSprite).SetPosition(PositionX, PositionY);
        PaintSprites;

        if iSleepSize > 0 then
          Sleep(iSleepSize);
      end;
    end
    else
      Application.ProcessMessages;

    // set pusher position
    (SpriteList[ANumberSites + Ord(tSEPusher)] as TSprite).SetPosition(PointTo.X, PointTo.Y);
    PaintSprites;

    if iStepSize <> 5 then
      if iSleepSize > 0 then
        Sleep(250 * iSleepSize);
  end;

  iCurrentPosition := NewPosition;

  Result := True;
end;

// pull device by pusher
function TFormVUH.PullDeviceByPusher: Boolean;
begin
  Result := False;

  if not DedicatedDriver.PullDeviceByPusher then
    Exit;

  if (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).PusherState = tPEmpty then
  begin
    (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).PusherState := tPFull;

    (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).Paint(BitmapMerged.Canvas);
    PaintBox.Canvas.Draw(0, 0, BitmapMerged);

    Result := True;
  end;
end;

// drop device by pusher
function TFormVUH.DropDeviceByPusher: Boolean;
begin
  Result := False;

  if not DedicatedDriver.DropDeviceByPusher then
    Exit;

  if (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).PusherState = tPFull then
  begin
    (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).PusherState := tPEmpty;

    (SpriteList[iNumberSites + Ord(tSEPusher)] as TPusherSprite).Paint(BitmapMerged.Canvas);
    PaintBox.Canvas.Draw(0, 0, BitmapMerged);

    Result := True;
  end;
end;

// insert device to socket
function TFormVUH.InsertDeviceToSocket(const Index: Integer): Boolean;
begin
  Result := False;

  if not DedicatedDriver.InsertDeviceToSocket(Index) then
    Exit;

  if (SpriteList[Index] as TSocketSprite).SocketState = tSEnabledEmpty then
  begin
    (SpriteList[Index] as TSocketSprite).SocketState := tSEnabledFull;
    PaintSocket(Index);

    Result := True;
  end;
end;

// remove device from socket
function TFormVUH.RemoveDeviceFromSocket(const Index: Integer): Boolean;
begin
  Result := False;

  if not DedicatedDriver.RemoveDeviceFromSocket(Index) then
    Exit;

  if (SpriteList[Index] as TSocketSprite).SocketState = tSEnabledFull then
  begin
    (SpriteList[Index] as TSocketSprite).SocketState := tSEnabledEmpty;
    PaintSocket(Index);

    Result := True;
  end;
end;

// get actuator status
function TFormVUH.GetActuatorStatus(const Index: Integer; const bPaint: Boolean = True): Integer;
var TextTemp: string;
begin
  Result := -1;

  if not SendCommand(TXT_CMD_IDX_SITE_ACTUATOR_STATUS, Index, TextTemp) then Exit;

  try
    Result := StrToInt(TextTemp);
  except
    on EConvertError do Result := 3;
  end;

  case Result of
    0:  // 0 ... Error actuator unit is not present
        (SpriteList[Index] as TSocketSprite).SocketSideState := tSSideHidden;
    1:  // 1 ... ZIF socket is opened by actuator
        (SpriteList[Index] as TSocketSprite).SocketSideState := tSSideEnabledDown;
    2:  // 2 ... ZIF socket is closed by actuator
        (SpriteList[Index] as TSocketSprite).SocketSideState := tSSideEnabledUp;
  else
    //   3 ... Communication error with actuator
    //   4 ... ZIF socket actuator is in unknown position, please re-configure
    (SpriteList[Index] as TSocketSprite).SocketSideState := tSSideDisabledUp;
  end;

  if bPaint then
    PaintSocket(Index);
end;

end.
