{
  filename - CTReelSprite.pas

  Purpose:
       This file contains sprite graphics class for device reel visualization.
}

unit CTReelSprite;
{
History (dd.mm.yy):
1.10.2010 start of development
}

interface

uses Graphics, Types,
     CTSprite;

type
  TReelSprite = class(TSprite)
  private
    Bitmap : TBitmap;
    QuarterLength, ThreeQuarterLength: Integer;
    HorizontalShift: Single; // animation reason
    FStatistics: Integer;
    DeltaMoving: Single;

  published
    property Statistics: Integer read FStatistics write FStatistics;

  public
    constructor Create(const Left, Top: Longint; const Bitmap: TBitmap);
    function GetCentrePosition(const PusherWidth: Integer): TPoint;
    procedure MoveFrame;
    procedure SetDeltaMoving(const StepSize: Integer);
    procedure Paint(const Canvas: TCanvas); override;
  end;

implementation

uses SysUtils;

constructor TReelSprite.Create(const Left, Top: Longint; const Bitmap: TBitmap);
begin
  inherited Create(Left, Top);

  Self.Bitmap := Bitmap;

  // 1/4 of image is visible
  QuarterLength := Bitmap.Width * 1 div 4;
  // 3/4 of image is visible
  ThreeQuarterLength := Bitmap.Width * 3 div 4;

  HorizontalShift := 0.0;

  FStatistics := 0;

  DeltaMoving := 0.5;
end;

function TReelSprite.GetCentrePosition(const PusherWidth: Integer): TPoint;
begin
  Result := GetPosition;
  // Frame = 1 / 4 Width of Bitmap (only three are visible)
  // centre position is 1 / 2 of second frame
  // there are correction according to pusher width
  Result.X := Result.X + Trunc(((Bitmap.Width * 1.0 / 4.0) * 1.5) - (PusherWidth / 2.0));
end;

procedure TReelSprite.MoveFrame;
begin
  HorizontalShift := QuarterLength;
end;

procedure TReelSprite.SetDeltaMoving(const StepSize: Integer);
begin
	DeltaMoving := StepSize / 2.0;
end;

procedure TReelSprite.Paint(const Canvas: TCanvas);
var Source, Destination: TRect;
begin
  Destination.Left   := Left;
  Destination.Top    := Top;
  Destination.Right  := Destination.Left + ThreeQuarterLength;
  Destination.Bottom := Destination.Top  + Bitmap.Height;

  Source.Left   := 0 + Trunc(HorizontalShift);
  Source.Top    := 0;
  Source.Right  := Source.Left + ThreeQuarterLength;
  Source.Bottom := Source.Top + Bitmap.Height;
  if HorizontalShift > 0.0 then
    HorizontalShift := HorizontalShift - DeltaMoving;

  Canvas.CopyRect(Destination, Bitmap.Canvas, Source);
  Canvas.TextOut(Left, Top + Bitmap.Height + 5, 'Counter:' + IntToStr(Statistics));
end;

end.
