{
  filename - CTSocketSprite.pas

  Purpose:
       This file contains sprite graphics class for programmer socket visualization.
}

unit CTSocketSprite;
{
History (dd.mm.yy):
 6. 2.2012 added matters about side socket
 1.10.2010 start of development
}

interface

uses Graphics,
     CTSprite, CTStatistics;

type
  TSocketState = (tSDisabledEmpty, tSEnabledEmpty, tSEnabledFull);
  TSocketSideState = (tSSideHidden, tSSideDisabledUp, tSSideEnabledUp, tSSideEnabledDown);

type
  TSocketSprite = class(TSprite)
  private
    BitmapDisabledEmpty: TBitmap;
    BitmapEnabledEmpty: TBitmap;
    BitmapEnabledFull: TBitmap;
    BitmapSideDisabledUp: TBitmap;
    BitmapSideEnabledUp: TBitmap;
    BitmapSideEnabledDown: TBitmap;
    FStatistics: TStatistics;
    FSocketState: TSocketState;
    FSocketSideState: TSocketSideState;
    FLedBusy, FLedGood, FLedError: Boolean;
    FSerialNumber: string;
    OrderNumber: integer;

  published
    property Statistics: TStatistics read FStatistics write FStatistics;
    property SocketState: TSocketState read FSocketState write FSocketState;
    property SocketSideState: TSocketSideState read FSocketSideState write FSocketSideState;
    property LedBusy: Boolean read FLedBusy write FLedBusy;
    property LedGood: Boolean read FLedGood write FLedGood;
    property LedError: Boolean read FLedError write FLedError;
    property SerialNumber: string read FSerialNumber write FSerialNumber;

  public
    constructor Create(const Left, Top: Longint; const OrderNumber: integer; const BitmapDisabledEmpty, BitmapEnabledEmpty, BitmapEnabledFull: TBitmap;
                       const BitmapSideDisabledUp, BitmapSideEnabledUp, BitmapSideEnabledDown: TBitmap);
    destructor Destroy; override;
    procedure Paint(const Canvas: TCanvas); override;
  end;

implementation

uses SysUtils, Classes;

constructor TSocketSprite.Create(const Left, Top: Longint; const OrderNumber: integer; const BitmapDisabledEmpty, BitmapEnabledEmpty, BitmapEnabledFull: TBitmap;
                                 const BitmapSideDisabledUp, BitmapSideEnabledUp, BitmapSideEnabledDown: TBitmap);
begin
  inherited Create(Left, Top);

  Self.OrderNumber := OrderNumber;
  FSocketState := tSDisabledEmpty;
  FSocketSideState := tSSideHidden;
  FLedBusy := False; FLedGood := False; FLedError := False;
  FStatistics := TStatistics.Create;
  Self.BitmapDisabledEmpty := BitmapDisabledEmpty;
  Self.BitmapEnabledEmpty := BitmapEnabledEmpty;
  Self.BitmapEnabledFull := BitmapEnabledFull;
  Self.BitmapSideDisabledUp := BitmapSideDisabledUp;
  Self.BitmapSideEnabledUp := BitmapSideEnabledUp;
  Self.BitmapSideEnabledDown := BitmapSideEnabledDown;

  FSerialNumber := '';
end;

destructor TSocketSprite.Destroy;
begin
  FStatistics.Free;

  inherited Destroy;
end;

procedure TSocketSprite.Paint(const Canvas: TCanvas);
const DISTANCE_SIZE = 15;

  procedure PaintLed(const IndexOrder: Integer; const EmitLight: Boolean; const EmittedColor, NotEmittedColor: TColor);
  begin
    if EmitLight then
      Canvas.Brush.Color := EmittedColor
    else
      Canvas.Brush.Color := NotEmittedColor;
    Canvas.Rectangle(Left + (IndexOrder * 2 * DISTANCE_SIZE), Top - 2 - DISTANCE_SIZE,
                     Left + (IndexOrder * 2 * DISTANCE_SIZE) + DISTANCE_SIZE, Top - 2);
  end;

var BrushColorOld: TColor;
    i: Integer;
    StatisticsList: TStrings;
    iVerticalIndent: integer;
begin
  BrushColorOld := Canvas.Brush.Color;

  iVerticalIndent := Top - (3 * DISTANCE_SIZE);
  Canvas.TextOut(Left, iVerticalIndent, '#' + IntToStr(OrderNumber));
  iVerticalIndent := Top - (2 * DISTANCE_SIZE);
  Canvas.TextOut(Left, iVerticalIndent, SerialNumber);
  iVerticalIndent := Top;

  // draw socket
  case FSocketState of
    tSDisabledEmpty: Canvas.Draw(Left, iVerticalIndent, BitmapDisabledEmpty);
    tSEnabledEmpty: Canvas.Draw(Left, iVerticalIndent, BitmapEnabledEmpty);
    tSEnabledFull: Canvas.Draw(Left, iVerticalIndent, BitmapEnabledFull);
  end;
  iVerticalIndent := iVerticalIndent + BitmapDisabledEmpty.Height;

  // draw side socket
  if FSocketSideState <> tSSideHidden then
  begin
    iVerticalIndent := iVerticalIndent + DISTANCE_SIZE;
    Canvas.FillRect(Rect(Left, iVerticalIndent,
                         Left + BitmapSideDisabledUp.Width, iVerticalIndent + BitmapSideDisabledUp.Height));
    case FSocketSideState of
      tSSideDisabledUp: Canvas.Draw(Left, iVerticalIndent, BitmapSideDisabledUp);
      tSSideEnabledUp: Canvas.Draw(Left, iVerticalIndent, BitmapSideEnabledUp);
      tSSideEnabledDown: Canvas.Draw(Left, iVerticalIndent, BitmapSideEnabledDown);
    end;
    iVerticalIndent := iVerticalIndent + BitmapSideDisabledUp.Height;
  end;

  // draw statistics
  StatisticsList := TStringList.Create;
  StatisticsList.Text := FStatistics.GetState;
  for i := 0 to StatisticsList.Count - 1 do
    Canvas.TextOut(Left, iVerticalIndent + (i * DISTANCE_SIZE) + 3, StatisticsList[i]);
  StatisticsList.Free;

  // draw LED
  PaintLed(0, FLedBusy, clRed, clMaroon);
  PaintLed(1, FLedGood, clLime, clGreen);
  PaintLed(2, FLedError, TColor($68CAFF), clOlive);

  Canvas.Brush.Color := BrushColorOld;
end;

end.

