{
  filename - CTStatistics.pas

  Purpose:
       This file contains statistics functions.
}

unit CTStatistics;
{
History (dd.mm.yy):
1.10.2010 start of development
}

interface

type
  TStatistics = class
    private
      Good,
      AdapterTestFailure, InsertionTestFailure, IDCheckFailure, AccessCanceledByUserFailure, OtherFailure {prog. SW, HW},
      OperationalFailure: Integer;

    public
      constructor Create;

      procedure Reset;
      procedure IncrementByResult(const ResultCode: integer);
      function GetGood: Integer;
      function GetErrors: Integer;
      function GetTotal: Integer;
      function GetState: string;
  end;

implementation

uses SysUtils;

constructor TStatistics.Create;
begin
  Reset;
end;

procedure TStatistics.Reset;
begin
  // set all counters to zero
  Good := 0;
  AdapterTestFailure := 0;
  InsertionTestFailure := 0;
  IDCheckFailure := 0;
  AccessCanceledByUserFailure := 0;
  OtherFailure := 0;
  OperationalFailure := 0;
end;

procedure TStatistics.IncrementByResult(const ResultCode: integer);
begin
  case ResultCode of
    // Unavailable
    0: ;
    // Good
    1: Inc(Good);
    // AdapterTestFailure
    2: Inc(AdapterTestFailure);
    // InsertionTestFailure
    3: Inc(InsertionTestFailure);
    // IDCheckFailure
    4: Inc(IDCheckFailure);
    // AccessCanceledByUserFailure
    5: Inc(AccessCanceledByUserFailure);
    // OtherFailure
    6: Inc(OtherFailure);
  else
    // OperationalFailure
    Inc(OperationalFailure);
  end;
end;

function TStatistics.GetGood: Integer;
begin
  Result := Good;
end;

function TStatistics.GetErrors: Integer;
begin
  Result := GetTotal - Good;
end;

function TStatistics.GetTotal: Integer;
begin
  Result := Good + InsertionTestFailure + IDCheckFailure + AccessCanceledByUserFailure +
            OtherFailure + OperationalFailure;
end;

function TStatistics.GetState: string;
const NEWLINE = #13#10;
begin
  Result := Format('G:  %5d' + NEWLINE + 'ADP:%5d' + NEWLINE + 'Ins:%5d' + NEWLINE + 'ID: %5d' + NEWLINE + 'ESC:%5d' + NEWLINE + 'OF: %5d' + NEWLINE + 'OPF:%5d',
                    [Good, AdapterTestFailure, InsertionTestFailure, IDCheckFailure, AccessCanceledByUserFailure, OtherFailure, OperationalFailure]);
end;

end.
