//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <Math.hpp>
#include "CTFormVUH.h"
#include "CTFormOperationalResult.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TFormVUH *FormVUH;
//---------------------------------------------------------------------------
#define TXT_TRUE "1"
#define TXT_FALSE "0"

#define TXT_CMD_PROTOCOL_VERSION      		"PROTOCOL VERSION"
#define TXT_CMD_PG4UWMC_VERSION       		"PG4UWMC VERSION"
#define TXT_CMD_MAX_INDEX_RANGE       		"SITES COUNT"
#define TXT_CMD_APPLICATION_CLOSE     		"APPLICATION CLOSE"
#define TXT_CMD_PROGRAMMING_ENVIRONMENT     "PROGRAMMING ENVIRONMENT"

#define TXT_CMD_SITES_STOP            		"SITES STOP"
#define TXT_CMD_SITES_LOAD_PROJECT    		"SITES LOAD PROJECT"

#define TXT_CMD_IDX_SITE_PRESENT      		"SITE PRESENT"  // state of site = APP_PRESENT and PGM_STATUS
#define TXT_CMD_IDX_SITE_BUSY         		"SITE BUSY"
#define TXT_CMD_IDX_SITE_RESULT       		"SITE RESULT"
#define TXT_CMD_IDX_SITE_START        		"SITE START"
#define TXT_CMD_IDX_SITE_SERIALNUMBER 		"SITE SERIALNUMBER"
#define TXT_CMD_IDX_SITE_MODULE_CHECK 		"SITE MODULE CHECK"
#define TXT_CMD_IDX_SITE_ACTUATOR_STATUS 	"SITE ACTUATOR STATUS"
#define TXT_CMD_IDX_SITE_ZIF_SOCKET_EMPTY 	"SITE ZIF SOCKET EMPTY"
#define TXT_CMD_IDX_SITES_STATISTICS      	"SITES STATISTICS"
#define TXT_CMD_IDX_SITE_STATISTICS       	"SITE STATISTICS"


#define TXT_CMD_UNKNOWN               		"UNKNOWN COMMAND"

#define TXT_HANDLER_RUNNING "Handler is still running!\nPlease wait a moment while handler correctly stop."

#define TXT_START "Start"
#define TXT_STOP "Stop"

#define TXT_MESSAGEBOX_TITLE "Virtual Universal Handler"

#define DEFAULT_NUMBER_SITES 8

#define TXT_MINIMUM_REQUIRED_PROTOCOL_VERSION "1.11"

void WriteToLogNoClass(const AnsiString Text)
{
	if (FormVUH != NULL)
		FormVUH->RichEditLogWindow->Lines->Add(Text);
}

__fastcall TFormVUH::TFormVUH(TComponent* Owner)
	: TForm(Owner)
{
	pSpriteList = new TObjectList();
	pDedicatedDriver = new TDedicatedDriver(&WriteToLogNoClass);

	iNumberSites = DEFAULT_NUMBER_SITES;

	iCurrentPosition = 0;
	iStepSize = 1;
	iSleepSize = 0;

	pBitmapMerged = new Graphics::TBitmap();

	pKeyboardCommandsHistory = new TStringList();

	FGUIControlState = tGCSBeforeInitialization;

	bFullManualMode = false;

	pFormLoadProject = new TFormLoadProject(this);
}

__fastcall TFormVUH::~TFormVUH()
{
	pSpriteList->Clear();
	delete pSpriteList;

	delete pDedicatedDriver;

	delete pBitmapMerged;

	delete pKeyboardCommandsHistory;

	delete pFormLoadProject;
}

void __fastcall TFormVUH::FormShow(TObject *Sender)
{
	pUniHandlerClient = NULL;

	// create Background Bitmap
	pBitmapMerged->Canvas->Brush->Color = clBtnFace;
	pBitmapMerged->Canvas->Font->Name = "Courier";
	//pBitmapMerged->Canvas->Handle = GetDC(PanelHandler->Handle);
	pBitmapMerged->Height = PaintBox->Height;
	pBitmapMerged->Width = PaintBox->Width;

	ReconfigureHandler();

	SetGUIControlState(tGCSBeforeInitialization);

	SetHandlerSpeed();
}

void __fastcall TFormVUH::FormClose(TObject *Sender, TCloseAction &Action)
{
	if (bFullManualMode == false)
		if (pUniHandlerClient != NULL)
		{
			delete pUniHandlerClient;
			pUniHandlerClient = NULL;
		}
}

void __fastcall TFormVUH::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	CanClose = (GUIControlState == tGCSBeforeInitialization) || (GUIControlState == tGCSAfterInitialization);
	if (!CanClose)
		Application->MessageBox(TXT_HANDLER_RUNNING, TXT_MESSAGEBOX_TITLE, MB_OK);
}

void __fastcall TFormVUH::FormPaint(TObject *Sender)
{
	if (pSpriteList->Count <= 0)
		return;

	// draw background
	pBitmapMerged->Height = PaintBox->Height;
	pBitmapMerged->Width = PaintBox->Width;
	pBitmapMerged->Canvas->FillRect(Rect(0, 0, PaintBox->Width, PaintBox->Height));
	// draw sprites
	for (int i = 0; i < pSpriteList->Count; i++)
		((TSprite *)(*pSpriteList)[i])->Paint(pBitmapMerged->Canvas);

	PaintBox->Canvas->Draw(0, 0, pBitmapMerged);
}

void __fastcall TFormVUH::PaintBoxPaint(TObject *Sender)
{
	Repaint();
}

#define TXT_PG4UWMC "PG4UWMC.exe"
void __fastcall TFormVUH::ButtonlServerPathClick(TObject *Sender)
{
	OpenDialogServerPath->Filter = "Program " +  (AnsiString)TXT_PG4UWMC + "|" + TXT_PG4UWMC;
	OpenDialogServerPath->Options << ofPathMustExist << ofFileMustExist << ofHideReadOnly;
	if (OpenDialogServerPath->Execute())
		EditServerPath->Text = OpenDialogServerPath->FileName;
}

void TFormVUH::AddParameterToCommandLine(const AnsiString NewParameter, AnsiString &Parameters)
{
	if (Parameters != "")
		Parameters = Parameters + " ";
	Parameters = Parameters + NewParameter;
}

void __fastcall TFormVUH::ButtonStartUpClick(TObject *Sender)
{
	#define	TXT_CMDLINE_AUTOCONNECTREMOTECONTROL "/autoconnectremotecontrol"
	#define	TXT_CMDLINE_DONOTLOADPROJECT "/donotloadproject"

	AnsiString FileNameWithPath, Parameters, sTextTemp;

	WriteToLog("Starting remote control server");

	FileNameWithPath = EditServerPath->Text;
	if (!FileExists(FileNameWithPath))
	{
		sTextTemp = "Server: '" + FileNameWithPath + "' does not exist.\n" +
								 "Please set correct server with path.";
		WriteToLog(sTextTemp);
		Application->MessageBox(sTextTemp.c_str(), TXT_MESSAGEBOX_TITLE, MB_OK | MB_ICONSTOP);
		return;
	}

	Parameters = "";
	if (CheckBoxAutoConnectRemoteControl->State == cbChecked)
		AddParameterToCommandLine(TXT_CMDLINE_AUTOCONNECTREMOTECONTROL, Parameters);
	if (CheckBoxDonotLoadProject->State == cbChecked)
		AddParameterToCommandLine(TXT_CMDLINE_DONOTLOADPROJECT, Parameters);

	if ((int)ShellExecute(this->Handle, NULL, ExtractFileName(FileNameWithPath).c_str(), Parameters.c_str(), ExtractFilePath(FileNameWithPath).c_str(), SW_SHOW) <= 32)
	{
		sTextTemp = "Application: '" + FileNameWithPath + "' can not be started.";
		WriteToLog(sTextTemp);
		Application->MessageBox(sTextTemp.c_str(), TXT_MESSAGEBOX_TITLE, MB_OK | MB_ICONSTOP);
		return;
	}
	else
		WriteToLog("Application: '" + FileNameWithPath + " " + Parameters + "' run successfully.");
}

void __fastcall TFormVUH::ButtonInitializeClick(TObject *Sender)
{
	if (bFullManualMode == false)
	{
		// stop of universal handler client
		if (pUniHandlerClient != NULL)
		{
			delete pUniHandlerClient;
			pUniHandlerClient = NULL;
			Sleep(100);  // wait for server
		}

		pUniHandlerClient = new TUniHandlerClient(&WriteToLogNoClass);
		if (pUniHandlerClient->Connect() == false)
			return;
	}

	AnsiString TextTemp;
	// mandatory as first instruction in code
	if (!SendCommand(TXT_CMD_PROTOCOL_VERSION, TextTemp))
		return;
	if ((AnsiString)TXT_MINIMUM_REQUIRED_PROTOCOL_VERSION > TextTemp.c_str())
	{
		WriteToLog("Minimum required protocol version (" + (AnsiString)TXT_MINIMUM_REQUIRED_PROTOCOL_VERSION +
							 ") is greater than the current running PG4UWMC protocol version (" + TextTemp + ").");

		WriteToLog("Please close and update PG4UWMC.");
		return;
	}

	if (!SendCommand(TXT_CMD_PG4UWMC_VERSION, TextTemp))
		return;

	if (!SendCommand(TXT_CMD_MAX_INDEX_RANGE, TextTemp))
		return;

	try
	{
		iNumberSites = StrToInt(TextTemp);
	}
	catch(EConvertError &exception)
	{
		iNumberSites = DEFAULT_NUMBER_SITES;
	}

	ReconfigureHandler();
	for (int i = 0; i < iNumberSites; i++)
	{
		// get actuator status
		if (GetActuatorStatus(i, False) == -1)
			return;

		// get site presence
		if (!SendCommand(TXT_CMD_IDX_SITE_PRESENT, i, TextTemp))
			return;
		if (TextTemp != TXT_TRUE)
			DisableSocket(i);
		else
			// site is present
			EnableSocket(i);

		Application->ProcessMessages();
	}
  Repaint();	

	// get sites serial number
	for (int i = 0; i < iNumberSites; i++)
	{
		if (!SendCommand(TXT_CMD_IDX_SITE_SERIALNUMBER, i, TextTemp))
			return;
		((TSocketSprite *)(*pSpriteList)[i])->SerialNumber = TextTemp;
	}

	// check modules in sites
	for (int i = 0; i < iNumberSites; i++)
	{
		if (!SendCommand(TXT_CMD_IDX_SITE_MODULE_CHECK, i, TextTemp))
			return;
		Application->ProcessMessages();
	}

	// check if ZIF socket is empty
	for (int i = 0; i < iNumberSites; i++)
	{
		if (!SendCommand(TXT_CMD_IDX_SITE_ZIF_SOCKET_EMPTY, i, TextTemp))
			return;
		Application->ProcessMessages();
	}

	Repaint();

	if (pDedicatedDriver->HandlerActive())
		SetGUIControlState(tGCSAfterInitialization);
}

void TFormVUH::AddNoneBlankValueParameterToCommandLine(const AnsiString NewParameter, const AnsiString ValueOfNewParameter, AnsiString &Parameters)
{
	if (ValueOfNewParameter != "")
	{
		if (Parameters != "")
			Parameters = Parameters + " ";
		Parameters = Parameters + NewParameter + ValueOfNewParameter;
	}
}

AnsiString TFormVUH::ReportProgrammingEnvironment(AnsiString Text)
{
	int iIndexInText;
	AnsiString sItem;

	AnsiString Result = "Report of programming environment:\n";

	do
	{
	  iIndexInText = Pos('|', Text);
	  if (iIndexInText > 0)
	  {
		sItem = Text.SubString(1, iIndexInText - 1);
		Text.Delete(1, iIndexInText);
		Result = Result + sItem + '\n';
	  }
	} while (Text != "");

	return Result;
}

void __fastcall TFormVUH::ButtonLoadProjectCheckModulesClick(TObject *Sender)
{
	AnsiString ProjectFileNameWithPath, sAnswer, sTextTemp;
	int	i, iAnswer;
	bool CheckModules;
	AnsiString OptionalParamaters, JobOperatorID, JobID, ProjectPassword, UniqueID;

	if (pFormLoadProject->ShowModal() == mrOk)
	{
		pFormLoadProject->GetParameter(CheckModules, ProjectFileNameWithPath, JobOperatorID, JobID, ProjectPassword, UniqueID);
		WriteToLog("Load project: " + ProjectFileNameWithPath);

		OptionalParamaters = "";
		AddNoneBlankValueParameterToCommandLine("/JobOperatorID:", JobOperatorID, OptionalParamaters);
		AddNoneBlankValueParameterToCommandLine("/JobID:", JobID, OptionalParamaters);
		AddNoneBlankValueParameterToCommandLine("/ProjectPassword:", ProjectPassword, OptionalParamaters);
		AddNoneBlankValueParameterToCommandLine("/UniqueID:", UniqueID, OptionalParamaters);
		if (OptionalParamaters != "")
			OptionalParamaters = " " + OptionalParamaters;
		if (!SendCommand(TXT_CMD_SITES_LOAD_PROJECT " " + ProjectFileNameWithPath + OptionalParamaters, sAnswer))
			return;

		// convert answer from string to integer
		try
		{
			iAnswer = StrToInt(sAnswer);
		}
		catch(EConvertError &exception)
		{
			iAnswer = 0;
		}

		// process answer of command
		switch (iAnswer)
		{
			case 0: sTextTemp = "Error: Unknown."; break;
			case 1: sTextTemp = "Project file load successful."; break;
			case 2: sTextTemp = "Error: Check 'Use Site #1 project for all Sites' in 'PG4UWMC Settings' dialog."; break;
			case 3: sTextTemp = "Error: Some site(s) is busy, stop programming operation before loading."; break;
			case 4: sTextTemp = "Error: Master site was not correctly started.\n"
											"       Try to do following steps in PG4UWMC:\n"
											"         1. disconnect Sites\n"
											"         2. connect Sites\n"
											"         3. load project."; break;
			case 5: sTextTemp = "Error: Failure during loading process."; break;
			case 6: sTextTemp = "Error: Bad project file password."; break;
			case 7: sTextTemp = "Error: Project load forbidden."; break;
			case 8: sTextTemp = "Project file load successful, but forced default \"Program\" operation."; break;
			case 9: sTextTemp = "Error: Bad project file unique ID."; break;
			default: sTextTemp = "Error: Unknown."; break;
		}
		WriteToLog(sTextTemp);
		if ((iAnswer != 1) && (iAnswer != 8))
		{
			Application->MessageBox(sTextTemp.c_str(), TXT_MESSAGEBOX_TITLE, MB_OK | MB_ICONWARNING);
			return;
		}
		else
		{
			// read and report programming environment
			if (SendCommand(TXT_CMD_PROGRAMMING_ENVIRONMENT, sAnswer))
				WriteToLog(ReportProgrammingEnvironment(sAnswer));
		}

		
		if (CheckModules)
		{
			// check modules in sites
			sTextTemp = "";
			for (i = 0; i < iNumberSites; i++)
			{
				if (!SendCommand(TXT_CMD_IDX_SITE_MODULE_CHECK, i, sAnswer))
					return;
				if (sAnswer != TXT_TRUE)
					if (sTextTemp != "")
						sTextTemp = sTextTemp + ", #" + IntToStr(i + 1);
					else
						sTextTemp = sTextTemp + "#" + IntToStr(i + 1);
				Application->ProcessMessages();
			}
			if (sTextTemp != "")
			{
				sTextTemp = "AP module(s) errors summary: " + sTextTemp + ".";
				WriteToLog(sTextTemp);
				Application->MessageBox(sTextTemp.c_str(), TXT_MESSAGEBOX_TITLE, MB_OK | MB_ICONWARNING);
			}
			else
				WriteToLog("AP module(s) check successful.");
		}
	}
}

void __fastcall TFormVUH::ButtonStartStopClick(TObject *Sender)
{
	if (GUIControlState == tGCSAfterInitialization)
	{
		SetGUIControlState(tGCSBeforeStart);

		// run Handler Logic Realization
		PostMessage(Handle, WM_USER_HANDLER_LOGIC_REALIZATION, 0, 0);
	}
	else
		SetGUIControlState(tGCSAfterStart);
}

void __fastcall TFormVUH::ButtonTotalStopClick(TObject *Sender)
{
	AnsiString sAnswer;

	WriteToLog("Total-Stop pressed.");
	if (!SendCommand(TXT_CMD_SITES_STOP, sAnswer))
  	return;
}

void __fastcall TFormVUH::ButtonCloseServerApplicationClick(TObject *Sender)
{
	AnsiString sAnswer, sTextTemp;
	int	iAnswer;

	WriteToLog("Close Application button pressed.");
	if (!SendCommand(TXT_CMD_APPLICATION_CLOSE, sAnswer))
		return;

	// convert answer from string to integer
	try
	{
		iAnswer = StrToInt(sAnswer);
	}
	catch (EConvertError &exception)
	{
		iAnswer = 0;
	}

	// process answer of command
	switch (iAnswer)
	{
		case 0: sTextTemp = "Error: Unknown."; break;
		case 1: sTextTemp = "Close application successful."; break;
		case 2: sTextTemp = "Error: Some site(s) is busy."; break;
		case 3: sTextTemp = "Error: Impossible to disconnect site(s)."; break;
		default: sTextTemp = "Error: Unknown."; break;
	}
	WriteToLog(sTextTemp);
	if (iAnswer != 1)
		Application->MessageBox(sTextTemp.c_str(), TXT_MESSAGEBOX_TITLE, MB_OK | MB_ICONWARNING);
}

void __fastcall TFormVUH::TrackBarSpeedChange(TObject *Sender)
{
	SetHandlerSpeed();
}

void __fastcall TFormVUH::EditCmdLineKeyUp(TObject *Sender, WORD &Key,
			TShiftState Shift)
{
	static int constIndexInKeyboardCommandsHistory = 0;
	AnsiString sAnswer;

  // set last command when UP ARROW key is press
	if (Key == VK_UP)
	{
		// select previous command from history
		if (pKeyboardCommandsHistory->Count > 0)
			if (constIndexInKeyboardCommandsHistory < pKeyboardCommandsHistory->Count)
			{
				constIndexInKeyboardCommandsHistory++;
				EditCmdLine->Text = (*pKeyboardCommandsHistory)[constIndexInKeyboardCommandsHistory - 1];
			}
		EditCmdLine->SelStart = EditCmdLine->Text.Length();
	}
  // set last command when DOWN ARROW key is press
	else if (Key == VK_DOWN)
	{
		// select following command from history
		if (pKeyboardCommandsHistory->Count > 0)
			if (constIndexInKeyboardCommandsHistory > 1)
			{
				constIndexInKeyboardCommandsHistory--;
				EditCmdLine->Text = (*pKeyboardCommandsHistory)[constIndexInKeyboardCommandsHistory - 1];
			}
		EditCmdLine->SelStart = EditCmdLine->Text.Length();
	}
	else if (Key == VK_RETURN)
	{
		if (!SendCommand(EditCmdLine->Text, sAnswer, true))
    	;

		// insert command to history
		if (pKeyboardCommandsHistory->Count == 0)
			pKeyboardCommandsHistory->Add(EditCmdLine->Text);
		else if ((*pKeyboardCommandsHistory)[0] != EditCmdLine->Text)
			pKeyboardCommandsHistory->Insert(0, EditCmdLine->Text);
		constIndexInKeyboardCommandsHistory = 0;

		// clear cmd line
		EditCmdLine->Text = "";
	}
}

void TFormVUH::SetHandlerSpeed()
{
	switch (TrackBarSpeed->Position)
	{
		case 1: iStepSize = 1; iSleepSize = 2; break;
		case 2: iStepSize = 1; iSleepSize = 1; break;
		case 3: iStepSize = 1; iSleepSize = 0; break;
		case 4: iStepSize = 2; iSleepSize = 0; break;
		case 5: iStepSize = 4; iSleepSize = 0; break;
		case 6: iStepSize = 5; iSleepSize = 0; break;		
		default: iStepSize = 1; iSleepSize = 0; break;
	}
	((TReelSprite *)(*pSpriteList)[iNumberSites + tSEReelIn])->SetDeltaMoving(iStepSize);
	((TReelSprite *)(*pSpriteList)[iNumberSites + tSEReelOutOK])->SetDeltaMoving(iStepSize);
	((TReelSprite *)(*pSpriteList)[iNumberSites + tSEReelOutBad])->SetDeltaMoving(iStepSize);
}

void TFormVUH::PaintSocket(const int Index)
{
	((TSocketSprite *)(*pSpriteList)[Index])->Paint(pBitmapMerged->Canvas);
	((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->Paint(pBitmapMerged->Canvas);
	PaintBox->Canvas->Draw(0, 0, pBitmapMerged);
}

void TFormVUH::EnableSocket(const int Index)
{
	if (((TSocketSprite *)(*pSpriteList)[Index])->SocketState == tSDisabledEmpty)
	{
		((TSocketSprite *)(*pSpriteList)[Index])->SocketState = tSEnabledEmpty;
		PaintSocket(Index);
	}
}

void TFormVUH::DisableSocket(const int Index)
{
	if (((TSocketSprite *)(*pSpriteList)[Index])->SocketState == tSDisabledEmpty)
		return;

	((TSocketSprite *)(*pSpriteList)[Index])->SocketState = tSDisabledEmpty;
	LedBusy(Index, false, false);
	LedGood(Index, false, false);
	LedError(Index, false, false);
	PaintSocket(Index);
}

TSocketState TFormVUH::GetSocketState(const int Index)
{
	return ((TSocketSprite *)(*pSpriteList)[Index])->SocketState;
}

bool TFormVUH::MoveReel(const TSpriteElements SpriteElement)
{
	switch (SpriteElement)
	{
		case tSEReelIn:
		case tSEReelOutOK:
		case tSEReelOutBad: break;
		default: return false;
	}

	((TReelSprite *)(*pSpriteList)[iNumberSites + SpriteElement])->MoveFrame();
	return true;
}

void TFormVUH::LedBusy(const int Index, const bool EmitLight, const bool Redraw)
{
	((TSocketSprite *)(*pSpriteList)[Index])->LedBusy = EmitLight;
	if (Redraw)
		PaintSocket(Index);
}

void TFormVUH::LedGood(const int Index, const bool EmitLight, const bool Redraw)
{
	((TSocketSprite *)(*pSpriteList)[Index])->LedGood = EmitLight;
	if (Redraw)
		PaintSocket(Index);
}

void TFormVUH::LedError(const int Index, const bool EmitLight, const bool Redraw)
{
	((TSocketSprite *)(*pSpriteList)[Index])->LedError = EmitLight;
	if (Redraw)
		PaintSocket(Index);
}

void TFormVUH::WriteToLog(const AnsiString Text)
{
	RichEditLogWindow->Lines->Add(Text);
}

void TFormVUH::ResetStatistics()
{
	((TReelSprite *)(*pSpriteList)[iNumberSites + tSEReelIn])->Statistics = 0;
	((TReelSprite *)(*pSpriteList)[iNumberSites + tSEReelOutOK])->Statistics = 0;
	((TReelSprite *)(*pSpriteList)[iNumberSites + tSEReelOutBad])->Statistics = 0;

	for (int i = 0; i < iNumberSites; i++)
		((TSocketSprite *)(*pSpriteList)[i])->Statistics.Reset();
}

void TFormVUH::IncrementReelStatistic(const TSpriteElements SpriteElement)
{
	switch (SpriteElement)
	{
		case tSEReelIn:
		case tSEReelOutOK:
		case tSEReelOutBad: break;
		default: return;
	}

	((TReelSprite *)(*pSpriteList)[iNumberSites + SpriteElement])->Statistics++;
}

void TFormVUH::IncrementSocketStatistic(const int Index, const int ResultCode)
{
	((TSocketSprite *)(*pSpriteList)[Index])->Statistics.IncrementByResult(ResultCode);
}

void TFormVUH::Finished(TMessage &Message)
{
	bool bCorrectFinish;

	bCorrectFinish = Message.WParam != 0;
	if (bCorrectFinish)
		SetGUIControlState(tGCSAfterInitialization);
	else
		SetGUIControlState(tGCSBeforeInitialization);
}

void TFormVUH::ReconfigureHandler()
{
	const int X0 = 10;
	const int Y0 = 10;

	pSpriteList->Clear();

	float SizeHorizontalElement = ImageSocketEnabledEmpty->Picture->Width * 1.3f;

	PaintBox->Width = X0 + (int)(Max(iNumberSites, 8) * SizeHorizontalElement);
	this->Constraints->MaxWidth = this->Width - ScrollBoxPaintBox->Width + PaintBox->Width + (2 * GetSystemMetrics(SM_CXEDGE));
	ScrollBoxPaintBox->Repaint();

  // adding sites to list
	for (int i = 0; i < iNumberSites; i++)
		pSpriteList->Add(new TSocketSprite(X0 + (int)(i * SizeHorizontalElement), 20 * Y0, i + 1,
									 ImageSocketDisabledEmpty->Picture->Bitmap, ImageSocketEnabledEmpty->Picture->Bitmap, ImageSocketEnabledFull->Picture->Bitmap,
									 ImageSocketSideDisabledUp->Picture->Bitmap, ImageSocketSideEnabledUp->Picture->Bitmap, ImageSocketSideEnabledDown->Picture->Bitmap));

	SizeHorizontalElement = ImageReelPanelIn->Picture->Width * 3.0f / 4.0f * 1.2f;
	// adding other elements to list
	// ReelIn
	pSpriteList->Add(new TReelSprite(X0 + (int)(tSEReelIn * SizeHorizontalElement), 1 * Y0, ImageReelPanelIn->Picture->Bitmap));
	// ReelOutOK
	pSpriteList->Add(new TReelSprite(X0 + (int)(tSEReelOutOK * SizeHorizontalElement), 1 * Y0, ImageReelPanelOutOK->Picture->Bitmap));
	// ReelOutBad
	pSpriteList->Add(new TReelSprite(X0 + (int)(tSEReelOutBad * SizeHorizontalElement), 1 * Y0, ImageReelPanelOutBad->Picture->Bitmap));
	// Pusher
	pSpriteList->Add(new TPusherSprite(X0 + (int)(tSEReelIn * SizeHorizontalElement), 1 * Y0, ImagePusherEmpty->Picture->Bitmap, ImagePusherFull->Picture->Bitmap));

	ResetStatistics();

	iCurrentPosition = iNumberSites + tSEReelIn;
}

void TFormVUH::SetGUIControlState(const TGUIControlState AGUIControlState)
{
	switch (AGUIControlState)
	{
		case tGCSBeforeInitialization:
													ComboBoxMode->Enabled = true;

													ButtonInitialize->Enabled = true;

													ButtonStartStop->Caption = TXT_START;
													ButtonStartStop->Enabled = false;
													PanelStart->Color = clGreen;

													ButtonLoadProjectCheckModules->Enabled = false;
													ButtonTotalStop->Enabled = false;
													ButtonCloseServerApplication->Enabled = false;
													PanelTotalStop->Color = clMaroon;

													LabelCmdLine->Enabled = false;
													EditCmdLine->Enabled = false;
													break;
		case tGCSAfterInitialization:
													ComboBoxMode->Enabled = true;

													ButtonInitialize->Enabled = true;

													ButtonStartStop->Caption = TXT_START;
													ButtonStartStop->Enabled = true;
													PanelStart->Color = clLime;

													ButtonLoadProjectCheckModules->Enabled = true;
													ButtonTotalStop->Enabled = true;
													ButtonCloseServerApplication->Enabled = true;
													PanelTotalStop->Color = clRed;

													LabelCmdLine->Enabled = true;
													EditCmdLine->Enabled = true;
													break;
		case tGCSBeforeStart:
													ComboBoxMode->Enabled = false;

													ButtonInitialize->Enabled = false;

													ButtonStartStop->Caption = TXT_STOP;
													ButtonStartStop->Enabled = true;
													PanelStart->Color = clLime;

													ButtonLoadProjectCheckModules->Enabled = false;
													ButtonTotalStop->Enabled = true;
													ButtonCloseServerApplication->Enabled = false;
													PanelTotalStop->Color = clRed;

													LabelCmdLine->Enabled = false;
													EditCmdLine->Enabled = false;
													break;
		case tGCSAfterStart:
													ComboBoxMode->Enabled = false;

													ButtonInitialize->Enabled = false;

													ButtonStartStop->Caption = TXT_STOP;
													ButtonStartStop->Enabled = false;
													PanelStart->Color = clGreen;

													ButtonLoadProjectCheckModules->Enabled = false;
													ButtonTotalStop->Enabled = true;
													ButtonCloseServerApplication->Enabled = false;
													PanelTotalStop->Color = clRed;

													LabelCmdLine->Enabled = false;
													EditCmdLine->Enabled = false;
													break;
		default: return;
	}
	FGUIControlState = AGUIControlState;
}

bool TFormVUH::SocketsEmpty()
{
	for (int i = 0; i < iNumberSites; i++)
		if (GetSocketState(i) == tSEnabledFull)
			return false;

	return true;
}

AnsiString TFormVUH::ReportStatistics(AnsiString Text)
{
	int iIndexInText, iStatisticValue;
	AnsiString sStatisticItem;
	int iStatisticSuccess, iStatisticAdapterTestFailure, iStatisticInsertionTestFailure, iStatisticIDCheckFailure,
		iStatisticAccessCanceledByUserFailure, iStatisticOtherFailure, iStatisticOperationalFail;

	iStatisticSuccess = 0;
	iStatisticAdapterTestFailure = 0;
	iStatisticInsertionTestFailure = 0;
	iStatisticIDCheckFailure = 0;
	iStatisticAccessCanceledByUserFailure = 0;
	iStatisticOtherFailure = 0;
	iStatisticOperationalFail = 0;

	do
	{
	  iIndexInText = Text.Pos('|');
	  if (iIndexInText > 0)
	  {
		sStatisticItem = Text.SubString(1, iIndexInText - 1);
		Text.Delete(1, iIndexInText);

		iIndexInText = sStatisticItem.Pos(':');
		if (iIndexInText > 0)
		{
		  iStatisticValue = StrToInt(sStatisticItem.SubString(iIndexInText + 1, sStatisticItem.Length()));
		  switch (StrToInt(sStatisticItem.SubString(1, iIndexInText - 1)))
		  {
			case 0: return " Unavailable statistics.";
			case 1: iStatisticSuccess = iStatisticValue;
					break;
			case 2: iStatisticAdapterTestFailure = iStatisticValue;
					break;
			case 3: iStatisticInsertionTestFailure = iStatisticValue;
					break;
			case 4: iStatisticIDCheckFailure = iStatisticValue;
					break;
			case 5: iStatisticAccessCanceledByUserFailure = iStatisticValue;
					break;
			case 6: iStatisticOtherFailure = iStatisticValue;
					break;
			default:iStatisticOperationalFail = iStatisticOperationalFail + iStatisticValue;
                	break;
		  }
		}

	  }
	} while (!Text.IsEmpty());

	return " Success:" + IntToStr(iStatisticSuccess) +
		   "  Oper. failure(s):" + IntToStr(iStatisticOperationalFail) +
		   "  Other failure(s):" + IntToStr(iStatisticAdapterTestFailure +
											iStatisticInsertionTestFailure +
											iStatisticIDCheckFailure +
											iStatisticAccessCanceledByUserFailure +
											iStatisticOtherFailure) +
		   "  Total:" + IntToStr(iStatisticSuccess +
								 iStatisticAdapterTestFailure +
								 iStatisticInsertionTestFailure +
								 iStatisticIDCheckFailure +
								 iStatisticAccessCanceledByUserFailure +
								 iStatisticOtherFailure +
								 iStatisticOperationalFail);
}

void TFormVUH::HandlerLogicRealization(TMessage &Message)
{
	static bool bSemaphore = true; // indicate running activity of procedure, protect procedure from multiple calling
	int iResultCode;
	AnsiString sCommandAnswer = "";

	if (!bSemaphore)
		return;

	bSemaphore = False;
	bool bCorrectFinish = False;

	// reset result flag for all sites
	// operation over each site of programmer
	for (int i = 0;  i < iNumberSites; i++)
		// CHECK RESULT OF SITE
		if (!SendCommand(TXT_CMD_IDX_SITE_RESULT, i, sCommandAnswer))
			return;

	try
	{
    // go to new device position
		if (!MovePusher(iNumberSites + tSEReelIn, iNumberSites)) return;

		// signalize request for initial loading of devices to sockets
		bool bInitialSocketsLoading = true;

		int LoadDevice = 0;

		// infinity main loop
		while ((GUIControlState == tGCSBeforeStart) || (!SocketsEmpty()))
		{
      Application->ProcessMessages();

      // operation over each site of programmer
			for (int i = 0; i < iNumberSites; i++)
			{
        // CHECK IF SITE IS PRESENT
				if (!SendCommand(TXT_CMD_IDX_SITE_PRESENT, i, sCommandAnswer))
					return;
				if (sCommandAnswer != TXT_TRUE)
				{
					DisableSocket(i);

					continue;
				}
        // site is present


				EnableSocket(i);

        // CHECK IF SITE IS BUSY
				if (!SendCommand(TXT_CMD_IDX_SITE_BUSY, i, sCommandAnswer))
					return;
				if (sCommandAnswer != TXT_FALSE)
				{
					LedBusy(i, true); LedGood(i, false); LedError(i, false);

					continue;
				}
				// site is not busy
        LedBusy(i, false);


        // CHECK RESULT OF SITE
				if (!SendCommand(TXT_CMD_IDX_SITE_RESULT, i, sCommandAnswer))
					return;

				try
				{
					iResultCode = StrToInt(sCommandAnswer);
				}
				catch(EConvertError &exception)
				{
					iResultCode = 0;
				}


				// load device
				if (bFullManualMode)
					if ((GUIControlState == tGCSBeforeStart) && (LoadDevice + 1 <= iNumberSites))
					{
						LoadDevice++;
						iResultCode = 0;
					}

				if (bInitialSocketsLoading)
				{
					// site result ... Unavailable
					if (iResultCode != 0)
						continue;
				}
				else // if (bInitialSocketsLoading)
				{
          // INCREMENT STATISTICS, REMOVE DEVICE FROM SOCKET AND INSERT IT TO OUTPUT REEL
          // site result ... Unavailable
					if (iResultCode == 0)
						continue;

					switch (GetActuatorStatus(i))
					{
						case 0: break;
						case 1: break;
						default: /*continue;*/ break;
					}

					if (ComboBoxMode->ItemIndex == 1)
					{
						TFormOperationalResult *pFormOperationalResult = new TFormOperationalResult(this);
						TPoint PointTemp;

						PointTemp.x = PanelControl->Left + 1;
						PointTemp.y = PanelControl->Top + 1;
						PointTemp = this->ClientToScreen(PointTemp);
						pFormOperationalResult->Left = PointTemp.x;
						pFormOperationalResult->Top = PointTemp.y;

						if (!MovePusher(i, iNumberSites)) return;

						pFormOperationalResult->SetButtonFocus(iResultCode == 1);
						if (pFormOperationalResult->ShowModal() == mrYes)
							iResultCode = 1; // Good
						else
							iResultCode = 7; // Operational Fail

						pFormOperationalResult->Release();
					}

          IncrementSocketStatistic(i, iResultCode);
					switch (iResultCode)
					{
						// site result ... Good
						case 1: LedGood(i, true); LedError(i, false); break;
						// site result ... Error
						default: LedGood(i, false); LedError(i, true); break;
					}

					// get statistics for site to log
					if (!SendCommand(TXT_CMD_IDX_SITE_STATISTICS, i, sCommandAnswer))
						return;
					else
						WriteToLog("Statistics info for Site#" + IntToStr(i + 1) + ReportStatistics(sCommandAnswer));

					if (GetSocketState(i) == tSEnabledFull)
					{
						if (!RemoveSocketSequence(i)) return;

						switch (iResultCode)
						{
							// site result ... Good
							case 1: if (!ReelOperationSequence(tSEReelOutOK))
												return;
											break;
							// site result ... Error
							default: if (!ReelOperationSequence(tSEReelOutBad))
												return;
											 break;
						}
					}
					else
						return;
				} // else // if (bInitialSocketsLoading)


				// TAKE NEW DEVICE, INSERT IT TO SOCKET AND START SITE
				if (GUIControlState == tGCSBeforeStart)
				{
					if (!(GetSocketState(i) == tSEnabledEmpty)) return;

					if (!ReelOperationSequence(tSEReelIn)) return;

					if (!InsertSocketSequence(i)) return;

					// start site #
					if (!SendCommand(TXT_CMD_IDX_SITE_START, i, sCommandAnswer))
						return;
					GetActuatorStatus(i);
					if (sCommandAnswer == TXT_FALSE)
						continue;

					// CHECK IF SITE IS BUSY (Busy Led correction)
					if (!SendCommand(TXT_CMD_IDX_SITE_BUSY, i, sCommandAnswer))
						return;
					if (sCommandAnswer == TXT_FALSE)
					{
						LedBusy(i, true); LedGood(i, false); LedError(i, false);
					}
				}
      }  // for (int i = 0; i < iNumberSites; i++)
			bInitialSocketsLoading = false;
		} // while ((GUIControlState == tGCSBeforeStart) || (!SocketsEmpty))

		// go to new device position
		if (!MovePusher(iNumberSites + tSEReelIn, iNumberSites)) return;

		// get total statistics to log
		if (!SendCommand(TXT_CMD_IDX_SITES_STATISTICS, sCommandAnswer))
		  return;
		else
		  WriteToLog("Summary:" + ReportStatistics(sCommandAnswer));

		bCorrectFinish = true;
	}
	__finally
	{
		if (!bCorrectFinish)
			WriteToLog("Error: Incorrect end of operation, analyse situation and continue by (Re)Initialization (recommended).");

		PostMessage(Handle, WM_USER_HANDLER_LOGIC_REALIZATION_FINISHED, bCorrectFinish, 0);
		bSemaphore = true;
	}
}

bool TFormVUH::SendCommand(const AnsiString Command, AnsiString &Answer, const Boolean FromCmdLine)
{
	if (pUniHandlerClient == NULL)
	{
		WriteToLog("Command: " + Command + " not accepted, (Re)Initialization is required.");
		return false;
	}

	if (pUniHandlerClient->Write(Command))
		if (pUniHandlerClient->Read(Answer))
		{
			WriteToLog(Command + " ... " + Answer);

			// successful finish
			return true;
		}
		else
		{
			if (Answer != "")
				WriteToLog(Command + " ... " + Answer);
		}

	if (!FromCmdLine)
		// incorrect finish
		SetGUIControlState(tGCSBeforeInitialization);

	return false;
}

bool TFormVUH::SendCommand(const AnsiString Command, const int Site, AnsiString &Answer)
{
	#define TXT_SPACE  " "

	return SendCommand(Command + TXT_SPACE + IntToStr(Site + 1), Answer);
}

// remove (insert) device from (to) input (output) reel by pusher
bool TFormVUH::ReelOperationSequence(const TSpriteElements SpriteElement)
{
	switch (SpriteElement)
	{
		case tSEReelIn:
		case tSEReelOutOK:
		case tSEReelOutBad: break;
		default: return false;
	}

	if (!MovePusher(iNumberSites + SpriteElement, iNumberSites)) return false;

	switch (SpriteElement)
	{
		case tSEReelIn: if (!PullDeviceByPusher()) return false;
										break;
		case tSEReelOutOK:
		case tSEReelOutBad: if (!DropDeviceByPusher()) return false;
												break;
	}

	if (!MoveReel(SpriteElement)) return false;
	IncrementReelStatistic(SpriteElement);

	return true;
}

// insert device to socket by pusher
bool TFormVUH::InsertSocketSequence(const int Index)
{
	// go to Index site
	if (!MovePusher(Index, iNumberSites)) return false;
	if (!InsertDeviceToSocket(Index)) return false;
	if (!DropDeviceByPusher()) return false;

	return true;
}

// remove device from socket by pusher
bool TFormVUH::RemoveSocketSequence(const int Index)
{
	// go to Index site
	if (!MovePusher(Index, iNumberSites)) return false;
	if (!RemoveDeviceFromSocket(Index)) return false;
	if (!PullDeviceByPusher()) return false;

	return true;
}

TPoint TFormVUH::GetPosition(const int Index, const int ANumberSites)
{
	switch (Index - ANumberSites)
	{
		case tSEReelIn:
		case tSEReelOutOK:
		case tSEReelOutBad: return ((TReelSprite *)(*pSpriteList)[Index])->GetCentrePosition(((TPusherSprite *)(*pSpriteList)[ANumberSites + tSEPusher])->GetWidth());
		default: return ((TSprite *)(*pSpriteList)[Index])->GetPosition();
	}
}

void TFormVUH::PaintSprites()
{
	// draw background
	pBitmapMerged->Canvas->FillRect(Rect(0, 0, PaintBox->Width, PaintBox->Height));
	// draw sprites
	for (int i = 0; i < pSpriteList->Count; i++)
		((TSprite *)(*pSpriteList)[i])->Paint(pBitmapMerged->Canvas);
	PaintBox->Canvas->Draw(0, 0, pBitmapMerged);
}

// move pusher from actual position to required NewPosition
bool TFormVUH::MovePusher(const int NewPosition, const int ANumberSites)
{
	int iCounter, PositionX, PositionY, iDelta, iDeltaAxisX, iDeltaAxisY, iDirectionAxisX, iDirectionAxisY;
	TPoint PointFrom, PointTo;

	if (pSpriteList->Count <= 0)
		return false;

	if (!pDedicatedDriver->MovePusher(NewPosition, iNumberSites)) return false;

	if (NewPosition != iCurrentPosition)
	{
		PointFrom = GetPosition(iCurrentPosition, ANumberSites);
    PointTo   = GetPosition(NewPosition, ANumberSites);

		if (iStepSize != 5)
		{
			// x,y axis movement
			iDeltaAxisX = abs(PointFrom.x - PointTo.x);
			iDeltaAxisY = abs(PointFrom.y - PointTo.y);
			iDelta = Max(iDeltaAxisX, iDeltaAxisY);
			if ((PointTo.x - PointFrom.x) > 0)
				iDirectionAxisX = 1;
			else
				iDirectionAxisX = -1;
			if ((PointTo.y - PointFrom.y) > 0)
				iDirectionAxisY = 1;
			else
				iDirectionAxisY = -1;

			iCounter = 0;
			while (iCounter < iDelta)
			{
				Application->ProcessMessages();
				iCounter += iStepSize;
				if (iDelta == iDeltaAxisX)
					PositionX = PointFrom.x + (iCounter * iDirectionAxisX);
				else
					PositionX = PointFrom.x + ((iCounter * iDirectionAxisX * iDeltaAxisX) / iDelta);
				if (iDelta == iDeltaAxisY)
					PositionY = PointFrom.y + (iCounter * iDirectionAxisY);
				else
					PositionY = PointFrom.y + ((iCounter * iDirectionAxisY * iDeltaAxisY) / iDelta);

				// set pusher position
				((TSprite *)(*pSpriteList)[ANumberSites + tSEPusher])->SetPosition(PositionX, PositionY);
				PaintSprites();

				if (iSleepSize > 0)
					Sleep(iSleepSize);
			}
		}
		else
			Application->ProcessMessages();


		// set pusher position
		((TSprite *)(*pSpriteList)[ANumberSites + tSEPusher])->SetPosition(PointTo.x, PointTo.y);
		PaintSprites();

		if (iStepSize != 5)
			if (iSleepSize > 0)
				Sleep(250 * iSleepSize);
	}

	iCurrentPosition = NewPosition;

  return true;
}

// pull device by pusher
bool TFormVUH::PullDeviceByPusher()
{
	if (!pDedicatedDriver->PullDeviceByPusher())
		return false;

	if (((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->PusherState == tPEmpty)
	{
		((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->PusherState = tPFull;

		((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->Paint(pBitmapMerged->Canvas);
		PaintBox->Canvas->Draw(0, 0, pBitmapMerged);
		return true;
	}
	return false;
}

// drop device by pusher
bool TFormVUH::DropDeviceByPusher()
{
	if (!pDedicatedDriver->DropDeviceByPusher())
		return false;

	if (((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->PusherState == tPFull)
	{
		((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->PusherState = tPEmpty;

		((TPusherSprite *)(*pSpriteList)[iNumberSites + tSEPusher])->Paint(pBitmapMerged->Canvas);
		PaintBox->Canvas->Draw(0, 0, pBitmapMerged);
		return true;
	}
	return false;
}

// insert device to socket
bool TFormVUH::InsertDeviceToSocket(const int Index)
{
	if (!pDedicatedDriver->InsertDeviceToSocket(Index))
		return false;

	if (((TSocketSprite*)(*pSpriteList)[Index])->SocketState == tSEnabledEmpty)
	{
		((TSocketSprite*)(*pSpriteList)[Index])->SocketState = tSEnabledFull;
		PaintSocket(Index);
		return true;
	}
	return false;
}

// remove device from socket
bool TFormVUH::RemoveDeviceFromSocket(const int Index)
{
	if (!pDedicatedDriver->RemoveDeviceFromSocket(Index))
		return false;

	if (((TSocketSprite*)(*pSpriteList)[Index])->SocketState == tSEnabledFull)
	{
		((TSocketSprite*)(*pSpriteList)[Index])->SocketState = tSEnabledEmpty;
		PaintSocket(Index);
		return true;
	}
	return false;
}

// get actuator status
int TFormVUH::GetActuatorStatus(const int Index, const bool bPaint)
{
	AnsiString TextTemp;

	int Result = -1;

	if (!SendCommand(TXT_CMD_IDX_SITE_ACTUATOR_STATUS, Index, TextTemp)) return Result;

	try
	{
		Result = StrToInt(TextTemp);
	}
	catch(EConvertError &exception)
	{
		Result = 3;
	}

	switch (Result)
	{
		case 0:  // 0 ... Error actuator unit is not present
						((TSocketSprite *)(*pSpriteList)[Index])->SocketSideState = tSSideHidden;
						break;
		case 1:  // 1 ... ZIF socket is opened by actuator
						((TSocketSprite *)(*pSpriteList)[Index])->SocketSideState = tSSideEnabledDown;
						break;
		case 2:  // 2 ... ZIF socket is closed by actuator
						((TSocketSprite *)(*pSpriteList)[Index])->SocketSideState = tSSideEnabledUp;
						break;
		default:
				//   3 ... Communication error with actuator
				//   4 ... ZIF socket actuator is in unknown position, please re-configure
				((TSocketSprite *)(*pSpriteList)[Index])->SocketSideState =  tSSideDisabledUp;
				break;
	}

	if (bPaint)
		PaintSocket(Index);

	return Result;
}

