
#pragma hdrstop

#include "CTReelSprite.h"

__fastcall TReelSprite::TReelSprite(const long int Left, const long int Top, Graphics::TBitmap *pBitmap)
	: TSprite(Left, Top)
{
	this->pBitmap = pBitmap;

	// 1/4 of image is visible
	QuarterLength = pBitmap->Width * 1 / 4;
	// 3/4 of image is visible
	ThreeQuarterLength = pBitmap->Width * 3 / 4;

	HorizontalShift = 0.0f;

	FStatistics = 0;

	DeltaMoving = 0.5f;
}

TPoint TReelSprite::GetCentrePosition(const int PusherWidth)
{
	TPoint Result = GetPosition();
	// Frame = 1 / 4 Width of Bitmap (only three are visible)
	// centre position is 1 / 2 of second frame
	// there are correction according to pusher width
	Result.x += (int)(((pBitmap->Width * 1.0f / 4.0f) * 1.5f) - (PusherWidth / 2.0f));
	return Result;
}

void TReelSprite::MoveFrame()
{
	HorizontalShift = QuarterLength;
}

void TReelSprite::SetDeltaMoving(const int StepSize)
{
	DeltaMoving = StepSize / 2.0;
}

void TReelSprite::Paint(TCanvas *pCanvas)
{
	TRect Source, Destination;

	Destination.Left   = Left;
	Destination.Top    = Top;
	Destination.Right  = Destination.Left + ThreeQuarterLength;
	Destination.Bottom = Destination.Top  + pBitmap->Height;

	Source.Left   = 0 + (int)HorizontalShift;
	Source.Top    = 0;
	Source.Right  = Source.Left + ThreeQuarterLength;
	Source.Bottom = Source.Top + pBitmap->Height;
	if (HorizontalShift > 0.0)
		HorizontalShift -= DeltaMoving;

	pCanvas->CopyRect(Destination, pBitmap->Canvas, Source);
	pCanvas->TextOut(Left, Top + pBitmap->Height + 5, "Counter:" + IntToStr(Statistics));
}

#pragma package(smart_init)
