
#pragma hdrstop

#include "CTSocketSprite.h"

__fastcall TSocketSprite::TSocketSprite(const long int Left, const long int Top, const int OrderNumber, Graphics::TBitmap *pBitmapDisabledEmpty, Graphics::TBitmap *pBitmapEnabledEmpty, Graphics::TBitmap *pBitmapEnabledFull,
																				Graphics::TBitmap *pBitmapSideDisabledUp, Graphics::TBitmap *pBitmapSideEnabledUp, Graphics::TBitmap *pBitmapSideEnabledDown)
	: TSprite(Left, Top)
{
	this->OrderNumber = OrderNumber;
	FSocketState = tSDisabledEmpty;
	FSocketSideState = tSSideHidden;
	FLedBusy = false; FLedGood = false; FLedError = false;
	FStatistics.Reset();
	this->pBitmapDisabledEmpty = pBitmapDisabledEmpty;
	this->pBitmapEnabledEmpty = pBitmapEnabledEmpty;
	this->pBitmapEnabledFull = pBitmapEnabledFull;
	this->pBitmapSideDisabledUp = pBitmapSideDisabledUp;
	this->pBitmapSideEnabledUp = pBitmapSideEnabledUp;
	this->pBitmapSideEnabledDown = pBitmapSideEnabledDown;

	FSerialNumber = "";
}

__fastcall TSocketSprite::~TSocketSprite()
{
}

void TSocketSprite::PaintLed(TCanvas *pCanvas, const int DistanceSize, const int IndexOrder, const bool EmitLight, const TColor EmittedColor, const TColor NotEmittedColor)
{
	if (EmitLight)
		pCanvas->Brush->Color = EmittedColor;
	else
		pCanvas->Brush->Color = NotEmittedColor;
	pCanvas->Rectangle(Left + (IndexOrder * 2 * DistanceSize), Top - 2 - DistanceSize,
										 Left + (IndexOrder * 2 * DistanceSize) + DistanceSize, Top - 2);
}

void TSocketSprite::Paint(TCanvas *pCanvas)
{
	const DISTANCE_SIZE = 15;

	TColor BrushColorOld = pCanvas->Brush->Color;

	int iVerticalIndent = Top - (3 * DISTANCE_SIZE);
	pCanvas->TextOut(Left, iVerticalIndent, "#" + IntToStr(OrderNumber));
	iVerticalIndent = Top - (2 * DISTANCE_SIZE);
	pCanvas->TextOut(Left, iVerticalIndent, SerialNumber);
	iVerticalIndent = Top;
	
	// draw socket
	switch (FSocketState)
	{
		case tSDisabledEmpty: pCanvas->Draw(Left, iVerticalIndent, pBitmapDisabledEmpty); break;
		case tSEnabledEmpty: pCanvas->Draw(Left, iVerticalIndent, pBitmapEnabledEmpty); break;
		case tSEnabledFull: pCanvas->Draw(Left, iVerticalIndent, pBitmapEnabledFull); break;
	}
	iVerticalIndent = iVerticalIndent + pBitmapDisabledEmpty->Height;

	// draw side socket
	if (FSocketSideState != tSSideHidden)
	{
		iVerticalIndent = iVerticalIndent + DISTANCE_SIZE;
		pCanvas->FillRect(Rect(Left, iVerticalIndent,
													 Left + pBitmapSideDisabledUp->Width, iVerticalIndent + pBitmapSideDisabledUp->Height));
		switch (FSocketSideState)
		{
			case tSSideDisabledUp: pCanvas->Draw(Left, iVerticalIndent, pBitmapSideDisabledUp); break;
			case tSSideEnabledUp: pCanvas->Draw(Left, iVerticalIndent, pBitmapSideEnabledUp); break;
			case tSSideEnabledDown: pCanvas->Draw(Left, iVerticalIndent, pBitmapSideEnabledDown); break;
		}
		iVerticalIndent = iVerticalIndent + pBitmapSideDisabledUp->Height;
	}

  // draw statistics
	TStrings *pStatisticsList = new TStringList();
	pStatisticsList->Text = FStatistics.GetState();
	for (int i = 0; i < pStatisticsList->Count; i++)
		pCanvas->TextOut(Left, iVerticalIndent + (i * DISTANCE_SIZE) + 3, (*pStatisticsList)[i]);
	delete pStatisticsList;

	// draw LED
	PaintLed(pCanvas, DISTANCE_SIZE, 0, FLedBusy, clRed, clMaroon);
	PaintLed(pCanvas, DISTANCE_SIZE, 1, FLedGood, clLime, clGreen);
	PaintLed(pCanvas, DISTANCE_SIZE, 2, FLedError, TColor(0x68CAFF), clOlive);

	pCanvas->Brush->Color = BrushColorOld;
}

#pragma package(smart_init)
