/*
	filename - CTSocketSprite.h

	Purpose:
			 This file contains sprite graphics class for programmer socket visualization.
*/
/*
History (dd.mm.yy):
 6. 2.2012 added matters about side socket
 1.10.2010 start of development
*/

#ifndef CTSocketSpriteH
#define CTSocketSpriteH

#include <Graphics.hpp>
#include "CTSprite.h"
#include "CTStatistics.h"

enum TSocketState { tSDisabledEmpty, tSEnabledEmpty, tSEnabledFull };
enum TSocketSideState { tSSideHidden, tSSideDisabledUp, tSSideEnabledUp, tSSideEnabledDown };

class TSocketSprite : public TSprite
{
private:	// User declarations
	Graphics::TBitmap *pBitmapDisabledEmpty;
	Graphics::TBitmap *pBitmapEnabledEmpty;
	Graphics::TBitmap *pBitmapEnabledFull;
	Graphics::TBitmap *pBitmapSideDisabledUp;
	Graphics::TBitmap *pBitmapSideEnabledUp;
	Graphics::TBitmap *pBitmapSideEnabledDown;
	TStatistics FStatistics;
	TSocketState FSocketState;
	TSocketSideState FSocketSideState;	
	bool FLedBusy, FLedGood, FLedError;
	AnsiString FSerialNumber;
	int OrderNumber;

	void PaintLed(TCanvas *pCanvas, const int DistanceSize, const int IndexOrder, const bool EmitLight, const TColor EmittedColor, const TColor NotEmittedColor);

public:		// User declarations
	__property TStatistics Statistics = {read=FStatistics, write=FStatistics};
	__property TSocketState SocketState = {read=FSocketState, write=FSocketState};
	__property TSocketSideState SocketSideState = {read=FSocketSideState, write=FSocketSideState};
	__property bool LedBusy = {read=FLedBusy, write=FLedBusy};
	__property bool LedGood = {read=FLedGood, write=FLedGood};
	__property bool LedError = {read=FLedError, write=FLedError};
	__property AnsiString SerialNumber = {read=FSerialNumber, write=FSerialNumber};

	__fastcall TSocketSprite(const long int Left, const long int Top, const int OrderNumber, Graphics::TBitmap *pBitmapDisabledEmpty, Graphics::TBitmap *pBitmapEnabledEmpty, Graphics::TBitmap *pBitmapEnabledFull,
													 Graphics::TBitmap *pBitmapSideDisabledUp, Graphics::TBitmap *pBitmapSideEnabledUp, Graphics::TBitmap *pBitmapSideEnabledDown);
	__fastcall ~TSocketSprite();

	void Paint(TCanvas *pCanvas);
};

#endif
