
#pragma hdrstop

#include "CTStatistics.h"

__fastcall TStatistics::TStatistics()
{
	Reset();
}

void TStatistics::Reset()
{
	// set all counters to zero
	Good = 0;
	AdapterTestFailure = 0;
	InsertionTestFailure = 0;
	IDCheckFailure = 0;
	AccessCanceledByUserFailure = 0;
	OtherFailure = 0;
	OperationalFailure = 0;
}

void TStatistics::IncrementByResult(const int ResultCode)
{
	switch (ResultCode)
	{
		// Unavailable
		case 0: break;
		// Good
		case 1: Good++; break;
		// AdapterTestFailure
		case 2: AdapterTestFailure++; break;
		// InsertionTestFailure
		case 3: InsertionTestFailure++; break;
		// IDCheckFailure
		case 4: IDCheckFailure++; break;
		// AccessCanceledByUserFailure
		case 5: AccessCanceledByUserFailure++; break;
		// OtherFailure
		case 6: OtherFailure++; break;
		// OperationalFailure
		default: OperationalFailure++; break;
	}
}

int TStatistics::GetGood()
{
	return Good;
}

int TStatistics::GetErrors()
{
	return GetTotal() - Good;
}

int TStatistics::GetTotal()
{
	return Good + InsertionTestFailure + IDCheckFailure + AccessCanceledByUserFailure +
						OtherFailure + OperationalFailure;
}

AnsiString TStatistics::GetState()
{
	TVarRec args[7] = { Good, AdapterTestFailure, InsertionTestFailure, IDCheckFailure, AccessCanceledByUserFailure, OtherFailure, OperationalFailure };
	return Format("G:  %5d\nADP:%5d\nIns:%5d\nID: %5d\nESC:%5d\nOF: %5d\nOPF:%5d", args, 7);
}

#pragma package(smart_init)
