﻿/*
	filename - ReelSprite.cs

	Purpose:
			 This file contains sprite graphics class for device reel visualization.
*/
/*
History (dd.mm.yy):
1.10.2010 start of development
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace VUH
{
    public class ReelSprite : Sprite
    {
        private Bitmap bitmap;
        private int quarterLength, threeQuarterLength;
        private float horizontalShift; // animation reason
        private int statistics;
        public int Statistics
        {
            get { return statistics; }
            set { statistics = value; }
        }
        private float DeltaMoving;

        public ReelSprite(int left, int top, Bitmap bitmap)
            : base(left, top)
        {
            this.bitmap = bitmap;

            // 1/4 of image is visible
            quarterLength = bitmap.Width * 1 / 4;
            // 3/4 of image is visible
            threeQuarterLength = bitmap.Width * 3 / 4;

            horizontalShift = 0f;

            statistics = 0;

            DeltaMoving = 0.5f;
        }
        public Point GetCentrePosition(int pusherWidth)
        {
            Point result = GetPosition();
            // Frame = 1 / 4 Width of Bitmap (only three are visible)
            // centre position is 1 / 2 of second frame
            // there are correction according to pusher width
            result.X += (int)(((bitmap.Width * 1f / 4f) * 1.5f) - (pusherWidth / 2f));
            return result;
        }
        public void MoveFrame()
        {
            horizontalShift = quarterLength;
        }
        public void SetDeltaMoving(int StepSize)
        {
	        DeltaMoving = StepSize / 2.0f;
        }
        public override void Paint(ref Graphics canvas)
        { 
            Rectangle source, destination;
            
            destination = new Rectangle(left, top, threeQuarterLength, bitmap.Height);
            source = new Rectangle(0 + (int)horizontalShift, 0, threeQuarterLength, bitmap.Height);
            if (horizontalShift > 0.0f)
                horizontalShift -= DeltaMoving;

            Font myFont = new System.Drawing.Font("Courier", 10, FontStyle.Regular);
            Brush myBrush = new SolidBrush(System.Drawing.Color.Black);
            canvas.DrawImage(bitmap, destination, source, GraphicsUnit.Pixel);
            canvas.DrawString("Counter:" + statistics.ToString(), myFont, myBrush, left, top + bitmap.Height + 5);
        }
    }
}
