﻿/*
	filename - SocketSprite.cs

	Purpose:
			 This file contains sprite graphics class for programmer socket visualization.
*/
/*
History (dd.mm.yy):
 6. 2.2012 added matters about side socket
 1.10.2010 start of development
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace VUH
{
    public enum EnumSocketState { tSDisabledEmpty, tSEnabledEmpty, tSEnabledFull };
    public enum EnumSocketSideState { tSSideHidden, tSSideDisabledUp, tSSideEnabledUp, tSSideEnabledDown };

    public class SocketSprite : Sprite
    {        
        Bitmap bitmapDisabledEmpty;
        Bitmap bitmapEnabledEmpty;
        Bitmap bitmapEnabledFull;
        Bitmap bitmapSideDisabledUp;
        Bitmap bitmapSideEnabledUp;
        Bitmap bitmapSideEnabledDown;

        Statistics statistics;
        public Statistics Statistics
        {
            get { return statistics; }
            set { statistics = value; }
        }
        EnumSocketState socketState;
        public EnumSocketState SocketState
        {
            get { return socketState; }
            set { socketState = value; }
        }
        EnumSocketSideState socketSideState;
        public EnumSocketSideState SocketSideState
        {
            get { return socketSideState; }
            set { socketSideState = value; }
        }
        bool ledBusy;
        public bool LedBusy
        {
            get { return ledBusy; }
            set { ledBusy = value; }
        }
        bool ledGood;
        public bool LedGood
        {
            get { return ledGood; }
            set { ledGood = value; }
        }
        bool ledError;
        public bool LedError
        {
            get { return ledError; }
            set { ledError = value; }
        }
        string serialNumber;
        public string SerialNumber
        {
            get { return serialNumber; }
            set { serialNumber = value; }
        }
        int orderNumber;

        public SocketSprite(int left, int top, int orderNumber, Bitmap bitmapDisabledEmpty, Bitmap bitmapEnabledEmpty, Bitmap bitmapEnabledFull,
                            Bitmap bitmapSideDisabledUp, Bitmap bitmapSideEnabledUp, Bitmap bitmapSideEnabledDown)
            : base(left, top)
        {
            this.orderNumber = orderNumber;
            socketState = EnumSocketState.tSDisabledEmpty;
            socketSideState = EnumSocketSideState.tSSideHidden;
            ledBusy = false; ledGood = false; ledError = false;
            statistics = new Statistics();
            this.bitmapDisabledEmpty = bitmapDisabledEmpty;
            this.bitmapEnabledEmpty = bitmapEnabledEmpty;
            this.bitmapEnabledFull = bitmapEnabledFull;
            this.bitmapSideDisabledUp = bitmapSideDisabledUp;
            this.bitmapSideEnabledUp = bitmapSideEnabledUp;
            this.bitmapSideEnabledDown = bitmapSideEnabledDown;
            this.bitmapSideDisabledUp.MakeTransparent();
            this.bitmapSideEnabledUp.MakeTransparent();
            this.bitmapSideEnabledDown.MakeTransparent();

            serialNumber = "";
        }
        void PaintLed(ref Graphics canvas, int distanceSize, int indexOrder, bool emitLight, Color emittedColor, Color notEmittedColor)
        {            
            Brush brush;
            if (emitLight)
                brush = new SolidBrush(emittedColor);
            else
                brush = new SolidBrush(notEmittedColor);
            canvas.FillRectangle(brush, left + (indexOrder * 2 * distanceSize), top - distanceSize - 2, distanceSize, distanceSize);
        }
        public override void Paint(ref Graphics canvas)
        {
            const int DISTANCE_SIZE = 15;

            Font myFont = new System.Drawing.Font("Courier", 10, FontStyle.Regular);
            Brush myBrush = new SolidBrush(System.Drawing.Color.Black);
            int iVerticalIndent = top - (3 * DISTANCE_SIZE) - 2;
            canvas.DrawString("#" + orderNumber.ToString(), myFont, myBrush, left, iVerticalIndent);
            iVerticalIndent = top - (2 * DISTANCE_SIZE) - 2;
            canvas.DrawString(serialNumber, myFont, myBrush, left, iVerticalIndent);
            iVerticalIndent = top;

            // draw socket
            switch(socketState)
            {
                case EnumSocketState.tSDisabledEmpty: canvas.DrawImage(bitmapDisabledEmpty, left, iVerticalIndent); break;
                case EnumSocketState.tSEnabledEmpty: canvas.DrawImage(bitmapEnabledEmpty, left, iVerticalIndent); break;
                case EnumSocketState.tSEnabledFull: canvas.DrawImage(bitmapEnabledFull, left, iVerticalIndent); break;
            }
            iVerticalIndent = iVerticalIndent + bitmapDisabledEmpty.Height;
            
            // draw side socket
            if (SocketSideState != EnumSocketSideState.tSSideHidden)
            {
                iVerticalIndent = iVerticalIndent + DISTANCE_SIZE;
                canvas.FillRectangle(new SolidBrush(SystemColors.ButtonFace), left, iVerticalIndent, left + bitmapSideDisabledUp.Width, iVerticalIndent + bitmapSideDisabledUp.Height);
                switch (SocketSideState)
                {
                    case EnumSocketSideState.tSSideDisabledUp: canvas.DrawImage(bitmapSideDisabledUp, left, iVerticalIndent); break;
                    case EnumSocketSideState.tSSideEnabledUp: canvas.DrawImage(bitmapSideEnabledUp, left, iVerticalIndent); break;
                    case EnumSocketSideState.tSSideEnabledDown: canvas.DrawImage(bitmapSideEnabledDown, left, iVerticalIndent); break;
                }
                iVerticalIndent = iVerticalIndent + bitmapSideDisabledUp.Height;
            }

            // draw statistics
            string [] statisticsList = Statistics.GetState().Split('\n');
            for (int i = 0; i < statisticsList.Count(); i++)
                canvas.DrawString(statisticsList[i], myFont, myBrush, left, iVerticalIndent + (i * DISTANCE_SIZE) + 3);

            // draw LED
            PaintLed(ref canvas, DISTANCE_SIZE, 0, ledBusy, Color.Red, Color.Maroon);
            PaintLed(ref canvas, DISTANCE_SIZE, 1, ledGood, Color.Lime, Color.Green);
            PaintLed(ref canvas, DISTANCE_SIZE, 2, ledError, Color.FromArgb(0xFF, 0xCA, 0x68), Color.Olive);
        }
    }
}
