﻿/*
	filename - Statistics.cs

	Purpose:
			 This file contains statistics functions.
*/
/*
History (dd.mm.yy):
1.10.2010 start of development
*/
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace VUH
{
    public class Statistics
    {
        private int good, adapterTestFailure, insertionTestFailure, iDCheckFailure, accessCanceledByUserFailure, otherFailure /*prog. SW, HW*/, 
                    operationalFailure;

        public Statistics()
        {
            Reset();
        }
        public void Reset()
        {
            // set all counters to zero
            good = 0;
            adapterTestFailure = 0;
            insertionTestFailure = 0;
            iDCheckFailure = 0;
            accessCanceledByUserFailure = 0;
            otherFailure = 0;
            operationalFailure = 0;
        }
        public void IncrementByResult(int resultCode)
        {
            switch (resultCode)
            {
                // Unavailable
                case 0: break;
                // Good
                case 1: good++; break;
                // AdapterTestFailure
                case 2: adapterTestFailure++; break;
                // InsertionTestFailure
                case 3: insertionTestFailure++; break;
                // IDCheckFailure
                case 4: iDCheckFailure++; break;
                // AccessCanceledByUserFailure
                case 5: accessCanceledByUserFailure++; break;
                // OtherFailure
                case 6: otherFailure++; break;
                // OperationalFailure
                default: operationalFailure++; break;
            }
        }
        public int GetGood()
        {
            return good;
        }
        public int GetErrors()
        {
            return GetTotal() - good;
        }
        public int GetTotal()
        {
            return good + insertionTestFailure + iDCheckFailure + accessCanceledByUserFailure +
                   otherFailure + operationalFailure;
        }
        public string GetState()
        {
            return String.Format("G:  {0,5}\nADP:{1,5}\nIns:{2,5}\nID: {3,5}\nESC:{4,5}\nOF: {5,5}\nOPF:{6,5}",
                                 good, adapterTestFailure, insertionTestFailure, iDCheckFailure, accessCanceledByUserFailure, otherFailure, operationalFailure);
        }
    }
}
